package es.gva.cit.catalogClient.protocols;

import java.beans.Beans;
import java.io.File;
import java.io.FileNotFoundException;
import java.net.URL;
import java.util.Hashtable;

import javax.swing.Spring;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.xerces.jaxp.DocumentBuilderFactoryImpl;
import org.apache.xerces.jaxp.DocumentBuilderImpl;
import org.jzkit.search.provider.iface.SearchException;
import org.jzkit.search.provider.iface.Searchable;
import org.jzkit.search.provider.z3950.Z3950ServiceFactory;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.factory.config.ConstructorArgumentValues;
import org.springframework.beans.factory.support.AbstractBeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionReaderUtils;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.beans.factory.xml.BeansDtdResolver;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.context.support.AbstractXmlApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;
import org.springframework.context.support.StaticApplicationContext;
import org.springframework.core.io.DefaultResourceLoader;
import org.springframework.web.context.support.StaticWebApplicationContext;
import org.springframework.web.context.support.XmlWebApplicationContext;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: Z3950ConnectionFactory.java,v 1.1.2.2 2006/11/15 00:08:08 jjdelcerro Exp $
 * $Log: Z3950ConnectionFactory.java,v $
 * Revision 1.1.2.2  2006/11/15 00:08:08  jjdelcerro
 * *** empty log message ***
 *
 * Revision 1.2  2006/10/02 08:29:07  jorpiell
 * Modificados los cambios del Branch 10 al head
 *
 * Revision 1.1.2.1  2006/09/20 12:01:07  jorpiell
 * Se ha cambiado la versin de la jzkit, se ha incorporado la funcionalidad de cargar arcims
 *
 * Revision 1.1  2006/09/20 11:20:17  jorpiell
 * Se ha cambiado la versin de la librera jzkit de la 1 a la 2.0
 *
 *
 */
/**
 * This factory creates Z39.50 connections
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class Z3950ConnectionFactory {
	private static ApplicationContext app_context = null;
	private static boolean appContextInitialized = false;
	private static Z3950ServiceFactory factory = null;
	private static Hashtable connections = new Hashtable();
	
	public static Z3950Connection getConnection(URL url){
		Z3950Connection connection = (Z3950Connection)connections.get(url);
		if (connection == null){
			connection = new Z3950Connection(url);
			connections.put(url,connection);
		}
		return connection;
	}
	
	public static Searchable getSearchable(URL url) throws SearchException{
		if (!(appContextInitialized)){
			initAppContext(url);
			factory = new Z3950ServiceFactory();		
			factory.setApplicationContext(app_context);
			factory.setDefaultRecordSyntax("usmarc");
			factory.setDefaultElementSetName("F");
			factory.getRecordArchetypes().put("Default","usmarc::F");
			factory.getRecordArchetypes().put("FullDisplay","usmarc::F");
			factory.getRecordArchetypes().put("BriefDisplay","usmarc::B");
			factory.getRecordArchetypes().put("Holdings","usmarc::F");
		}
		factory.setHost(url.getHost());
		factory.setPort(url.getPort());
		Searchable s = factory.newSearchable();
		s.setApplicationContext(app_context);	
		return s;
	}

	/**
	 * Initialize the application context
	 * @param url 
	 */
	private static void initAppContext(URL url){
		System.setProperty("javax.xml.parsers.DocumentBuilderFactory","org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
		try{
			app_context = new FileSystemXmlApplicationContext(new String[]{"config/ApplicationContextAlone.xml"},false);
			((FileSystemXmlApplicationContext)app_context).setClassLoader(Z3950ConnectionFactory.class.getClassLoader());
			((FileSystemXmlApplicationContext)app_context).refresh();
		}catch(Exception e){
			app_context = new FileSystemXmlApplicationContext(new String[]{"gvSIG/extensiones/es.gva.cit.gvsig.catalogClient/config/ApplicationContext.xml"},false);
			((FileSystemXmlApplicationContext)app_context).setClassLoader(Z3950ConnectionFactory.class.getClassLoader());
			((FileSystemXmlApplicationContext)app_context).refresh();
		}			
		
		if (app_context == null){
			throw new RuntimeException("Unable to locate TestApplicationContext.xml definition file");
		}
		appContextInitialized = true;
	}
	
}
