
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package es.gva.cit.gazetteer.wfsg.parsers;
import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.metadataXML.XMLTree;
import es.gva.cit.catalogClient.querys.Coordinates;
import es.gva.cit.gazetteer.querys.ThesaurusName;
import es.gva.cit.gazetteer.wfsg.drivers.WFSGazetteerServiceDriver;
import java.util.Vector;

/**
 * This class parses a WFs-G getCapabilities file
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class WfsgCapabilitiesParser {
	/**
	 * 
	 * 
	 */
	WFSGazetteerServiceDriver driver;
	/**
	 * 
	 * 
	 */
	XMLNode rootNode = null;

	/**
	 * 
	 * 
	 * 
	 * @param driver 
	 */
	public  WfsgCapabilitiesParser(WFSGazetteerServiceDriver driver) {        
		this.driver = driver;
	} 

	/**
	 * 
	 * 
	 * 
	 * @return 
	 * @param node 
	 */
	public boolean parse(XMLNode node) {        
		if ((node == null) || !(node.getName().equals("WFS_Capabilities"))){
			driver.setServerAnswerReady("El servidor no soporta el protocolo " +
			"especificado");
			return false;
		}


		setRootNode(node);

		driver.setServerAnswerReady(XMLTree.searchNodeValue(node,
		"Service->Title") + "\n" +
		XMLTree.searchNodeValue(node,"Service->Abstract"));

		parseGetCapabilities();
		parseDescribeFeatureType();
		parseGetFeature();
		parseFeatures();
		parseProjection();
		return true;
	} 

	/**
	 * Parses the server projection
	 */
	public void parseProjection(){
		driver.setProjection(XMLTree.searchNodeValue(getRootNode(),"FeatureTypeList->FeatureType->SRS"));
	}
	/**
	 * It parses the getCapabilities operation options
	 * 
	 */
	public void parseGetCapabilities() {        
		driver.setOperations(new WfsgProtocolsOperations());
		driver.getOperations().setGetCapabilities(getOperations("GetCapabilities"));
	} 

	/**
	 * It parses the DescribeFeatureType operation options
	 * 
	 */
	public void parseDescribeFeatureType() {        
		driver.getOperations().setDescribeFeatureType(getOperations("DescribeFeatureType"));
	} 

	/**
	 * It parses the GetFeature operation options
	 * 
	 */
	public void parseGetFeature() {        
		driver.getOperations().setGetFeature(getOperations("GetFeature"));
	} 

	/**
	 * It parses the Feature Types
	 * 
	 */
	public void parseFeatures() {        
		XMLNode[] features = XMLTree.searchMultipleNode(getRootNode(),"FeatureTypeList->FeatureType");
		Vector v = new Vector();
		for (int i=0 ; i<features.length ; i++){
			v.add(parseFeature(features[i]));
		}
		driver.setVectorFeatures(v);

	} 

	/**
	 * It parses a Feature Node.
	 * 
	 * 
	 * @return 
	 * @param featureNode 
	 */
	public ThesaurusName parseFeature(XMLNode featureNode) {        
		ThesaurusName f = new ThesaurusName();
		f.setName(XMLTree.searchNodeValue(featureNode,"Name"));
		f.setTitle(XMLTree.searchNodeValue(featureNode,"Title"));
		if (f.getTitle() == null)
			f.setTitle(f.getName());
		f.setAbstract(XMLTree.searchNodeValue(featureNode,"Abstract"));
		f.setKeywords(XMLTree.searchNodeValue(featureNode,"Keywords"));
		f.setSrs(XMLTree.searchNodeValue(featureNode,"SRS"));
		f.setCoordinates(new Coordinates(XMLTree.searchNodeAtribute(featureNode,"LatLongBoundingBox","minx"),
				XMLTree.searchNodeAtribute(featureNode,"LatLongBoundingBox","miny"),
				XMLTree.searchNodeAtribute(featureNode,"LatLongBoundingBox","maxx"),
				XMLTree.searchNodeAtribute(featureNode,"LatLongBoundingBox","maxy")));

		return f;
	} 

	/**
	 * This function parses the protocols of an operation
	 * 
	 * 
	 * @return 
	 * @param operation Operation to find the supported protocols
	 */
	public String[] getOperations(String operation) {        
		XMLNode[] protocols = XMLTree.searchMultipleNode(getRootNode(),"Capability->Request->" + operation + "->DCPType");
		Vector vProtocols = new Vector();

		for (int i=0 ; i<protocols.length ; i++){
			if (XMLTree.searchNode(protocols[i],"HTTP->Get") != null){
				vProtocols.add("GET");
			}
			if (XMLTree.searchNode(protocols[i],"HTTP->Post") != null){
				vProtocols.add("POST");
			}

		}  

		String[] sProtocols = new String[vProtocols.size()];
		for (int i=0 ; i<vProtocols.size() ; i++){
			sProtocols[i] = (String) vProtocols.get(i);
		}        

		return sProtocols;
	} 

	/**
	 * 
	 * 
	 * 
	 * @return Returns the rootNode.
	 */
	public XMLNode getRootNode() {        
		return rootNode;
	} 

	/**
	 * 
	 * 
	 * 
	 * @param rootNode The rootNode to set.
	 */
	public void setRootNode(XMLNode rootNode) {        
		this.rootNode = rootNode;
	} 
}
