/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig;

import java.awt.Component;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.util.Date;
import java.util.prefs.Preferences;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.ValidationException;

import com.iver.andami.Launcher;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.Messages;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.ProjectFactory;
import com.iver.cit.gvsig.project.documents.exceptions.OpenException;
import com.iver.cit.gvsig.project.documents.gui.ProjectWindow;
import com.iver.cit.gvsig.project.documents.layout.ProjectMap;
import com.iver.cit.gvsig.project.documents.layout.ProjectMapFactory;
import com.iver.cit.gvsig.project.documents.table.ProjectTableFactory;
import com.iver.cit.gvsig.project.documents.view.ProjectView;
import com.iver.cit.gvsig.project.documents.view.ProjectViewFactory;
import com.iver.utiles.GenericFileFilter;
import com.iver.utiles.XMLEntity;
import com.iver.utiles.extensionPoints.ExtensionPoint;
import com.iver.utiles.extensionPoints.ExtensionPoints;
import com.iver.utiles.extensionPoints.ExtensionPointsSingleton;
import com.iver.utiles.xml.XMLEncodingUtils;
import com.iver.utiles.xmlEntity.generate.XmlTag;


/**
 * Extension que proporciona controles para crear proyectos nuevos, abrirlos y
 * guardarlos. Adems los tipos de tabla que soporta el proyecto son aadidos
 * en esta clase.
 *
 * @author Fernando Gonzlez Corts
 */
public class ProjectExtension extends Extension {
	private static String projectPath = null;
	//private ProjectWindow projectFrame;
	private ProjectWindow projectFrame;
	private Project p;
	private String lastPath;
	private String lastSavePath;
	private String templatesPath;
	private WindowInfo seedProjectWindow;
	/**
	 * Use UTF-8 for encoding, as it can represent characters from
	 * any language.
	 *
	 * Another sensible option would be
	 * encoding = System.getProperty("file.encoding");
	 * but this would need some extra testing.
	 */
	private String projectEncoding = "UTF-8";

	/**
	 * @see com.iver.mdiApp.plugins.IExtension#initialize()
	 */
	public void initialize() {
	    try {
            Class.forName("javax.media.jai.EnumeratedParameter");
        } catch (ClassNotFoundException e) {
            NotificationManager.addError("La mquina virtual con la que se ejecuta gvSIG no tiene JAI instalado", e);
        }

		LayerFactory.setWritersPath(PluginServices.getPluginServices(this)
				  .getPluginDirectory()
				  .getAbsolutePath() +
				  File.separator + "drivers");

        LayerFactory.setDriversPath(PluginServices.getPluginServices(this)
				  .getPluginDirectory()
				  .getAbsolutePath() +
				  File.separator + "drivers");

        initializeDocumentActionsExtensionPoint();
        registerDocuments();

        // Recuperamos el ltimo argumento, que se supone
        // que ser el fichero .gvp que queremos abrir.
        // (por enmedio pueden venir o no otros argumentos,
        // por ejemplo el idioma)
        // TODO: Aqu Jaume podra meter lo del backup del proyecto
        // que ha hecho para ValenciaUrbanstica

		/*
		pe = (ProjectExtension) PluginServices.getExtension(com.iver.cit.gvsig.ProjectExtension.class);
		if (projectFile.length()!=0){
			p=pe.readProject(projectFile);
		}
		else
			p = restorePreviousProject();
		pe.setProject(p); */
        String[] theArgs = PluginServices.getArguments();
        String lastArg = theArgs[theArgs.length-1];
        if ((lastArg.endsWith(".gvp")) ||
        		(lastArg.endsWith(".GVP")))
        {
        	PluginServices.getLogger().debug("Intentando cargar el proyecto " + lastArg);
        	//File projectFile = new File(lastArg);
        	p = readProject(lastArg);
        	PluginServices.getMainFrame().setTitle(p.getName());
        	projectPath = lastArg;
        }
        else
        {
			p = ProjectFactory.createProject();
			p.setName(PluginServices.getText(this, "untitled"));
			p.setModified(false);
			PluginServices.getMainFrame().setTitle(PluginServices.getText(this, "sin_titulo"));
        }
	}
	/**
	 * @see com.iver.mdiApp.plugins.IExtension#postInitialize()
	 */
	public void postInitialize() {
		getProjectFrame().setProject(p);
		p.restoreWindowProperties();
	}

	private ProjectWindow getProjectFrame() {
		if (projectFrame==null)
				projectFrame = new ProjectWindow();
		return projectFrame;
	}
	/**
	 * Muestra la ventana con el gestor de proyectos.
	 */
	public void showProjectWindow() {
		if (seedProjectWindow!=null) {
			if (seedProjectWindow.isClosed()) {
				// if it was closed, we just don't open the window now
				seedProjectWindow.setClosed(false);
				return;
			}
			WindowInfo winProps = seedProjectWindow;
			seedProjectWindow = null;
			PluginServices.getMDIManager().addWindow(getProjectFrame());
			PluginServices.getMDIManager().changeWindowInfo(getProjectFrame(), winProps);
		}
		else
			PluginServices.getMDIManager().addWindow(getProjectFrame());
	}

	/**
	 * Muestra la ventana con el gestor de proyectos, con las propiedades
	 * de ventana especificadas.
	 */
	public void showProjectWindow(WindowInfo wi) {
		seedProjectWindow = wi;
		showProjectWindow();
	}

	/**
	 * Guarda el proyecto actual en disco.
	 */
	private boolean guardar() {
		boolean saved=false;
//		if (p.getPath() == null) {
		if (projectPath == null) {
			saved=guardarDialogo();
		} else {
			saved=writeProject(new File(projectPath), p); //new File(p.getPath()), p);
			getProjectFrame().refreshControls();
		}
		return saved;
	}

	private boolean guardarDialogo(){
		boolean saved=false;

		if (lastSavePath == null)
			lastSavePath = projectPath;

		if (lastSavePath == null) {
			Preferences prefs = Preferences.userRoot().node( "gvsig.foldering" );
			lastSavePath = prefs.get("ProjectsFolder", null);
		}

		JFileChooser jfc = new JFileChooser(lastSavePath);
		jfc.addChoosableFileFilter(new GenericFileFilter("gvp",
				PluginServices.getText(this, "tipo_fichero_proyecto")));

		if (jfc.showSaveDialog((Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION) {
			File file=jfc.getSelectedFile();
			if (!(file.getPath().toLowerCase().endsWith(".gvp"))){
				file=new File(file.getPath()+".gvp");
			}
			saved=writeProject(file, p);
			String filePath = file.getAbsolutePath();
			lastSavePath = filePath.substring(0, filePath.lastIndexOf(File.separatorChar));

			getProjectFrame().refreshControls();
		}
		return saved;
	}
	/**
	 * Guarda si el proyecto ha sido modificado.
	 *
	 * @return True si se ha guardado correctamente.
	 */
	private boolean modificado() {
		if (p.isModified()) {
		///if (true) {
			//TODO de momento se queda como modificado siempre menos cuando est totalmente vacio que se tomar como no modificado,
			//para poder controlar perfectamente cuando un proyecto ha sido modificado
			//hay que recoger los eventos de cambio de leyenda, cambio de extent, aadir una capa, etc que se encuentran en FMap.

			int res = JOptionPane.showConfirmDialog((Component) PluginServices.getMainFrame(),
					PluginServices.getText(this, "guardar_cambios"),
					"gvSIG",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.INFORMATION_MESSAGE);

			if (res == JOptionPane.YES_OPTION) {
				guardar();
			} else if (res == JOptionPane.CANCEL_OPTION) {
				return false;
			}
		}

		return true;
	}

	/**
	 * @see com.iver.mdiApp.plugins.IExtension#updateUI(java.lang.String)
	 */
	public void execute(String actionCommand) {
		if (actionCommand.equals("NUEVO")) {
			//Si est modificado se pregunta si se quiere guardar el anterior
			if (!modificado()) {
				return;
			}
			ProjectView.numViews=0;
            ProjectMap.numMaps = 0;
			PluginServices.getMDIManager().closeAllWindows();
			p = ProjectFactory.createProject();
			p.setName(PluginServices.getText(this, "untitled"));
			p.setModified(false);
			getProjectFrame().setProject(p);
			showProjectWindow();
			PluginServices.getMainFrame().setTitle(PluginServices.getText(this, "sin_titulo"));
		} else if (actionCommand.equals("ABRIR")) {
			//Si est modificado se pregunta si se quiere guardar el anterior
			if (!modificado()) {
				ProjectView.numViews=0;
                ProjectMap.numMaps = 0;
				return;
			}

			if (lastPath == null) {
				Preferences prefs = Preferences.userRoot().node( "gvsig.foldering" );
				lastPath = prefs.get("ProjectsFolder", null);
			}

			JFileChooser jfc = new JFileChooser(lastPath);
			jfc.addChoosableFileFilter(new GenericFileFilter("gvp",
					PluginServices.getText(this, "tipo_fichero_proyecto")));

			if (jfc.showOpenDialog((Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION) {
				ProjectView.numViews=0;
                ProjectMap.numMaps = 0;
				PluginServices.getMDIManager().closeAllWindows();

				File projectFile = jfc.getSelectedFile();
				Project o = readProject(projectFile);
				setPath(projectFile.getAbsolutePath());
				lastPath = getPath();
				if (o != null) {
					p = o;
				}

				getProjectFrame().setProject(p);
				PluginServices.getMainFrame().setTitle(p.getName());
				getProjectFrame().refreshControls();
				p.restoreWindowProperties();
			}
		} else if (actionCommand.equals("GUARDAR")) {
			guardar();
		} else if (actionCommand.equals("GUARDAR_COMO")) {
			guardarDialogo();
		} else if (actionCommand.equals("SALIR")){
			int option = JOptionPane.showConfirmDialog((Component)PluginServices.getMainFrame(),
					Messages.getString("MDIFrame.quiere_salir"),
					Messages.getString("MDIFrame.salir"),
					JOptionPane.YES_NO_OPTION);

			if (option == JOptionPane.YES_OPTION) {
				Launcher.closeApplication();
			}
		} else if (actionCommand.compareTo("OPENTEMPLATE")==0){
			openLayout();
		}
	}
	public void openLayout() {
		//Project project = ((ProjectExtension) PluginServices.getExtension(ProjectExtension.class)).getProject();
		Layout layout=null;

		if (templatesPath == null) {
			Preferences prefs = Preferences.userRoot().node( "gvsig.foldering" );
			templatesPath = prefs.get("TemplatesFolder", null);
		}

		JFileChooser jfc = new JFileChooser(templatesPath);
		jfc.addChoosableFileFilter(new GenericFileFilter("gvt",
				PluginServices.getText(this, "plantilla")));

		if (jfc.showOpenDialog((Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION) {
			File file=jfc.getSelectedFile();
			if (!(file.getPath().endsWith(".gvt") || file.getPath().endsWith(".GVT"))){
				file=new File(file.getPath()+".gvt");
			}
			try {
				File xmlFile = new File(file.getAbsolutePath());
				FileReader reader;
				reader = new FileReader(xmlFile);

				XmlTag tag = (XmlTag) XmlTag.unmarshal(reader);
				try {
					layout = Layout.createLayout(new XMLEntity(tag),p);
				} catch (OpenException e) {
					e.showError();
				}
				//fPanelLegendManager.setRenderer(LegendFactory.createFromXML(new XMLEntity(tag)));
			} catch (FileNotFoundException e) {
				NotificationManager.addError(PluginServices.getText(this, "Al_leer_la_leyenda"), e);
			} catch (MarshalException e) {
				NotificationManager.addError(PluginServices.getText(this, "Al_leer_la_leyenda"), e);
			} catch (ValidationException e) {
				NotificationManager.addError(PluginServices.getText(this, "Al_leer_la_leyenda"), e);
			}
			ProjectMap pmap = ProjectFactory.createMap(file.getName());
			pmap.setModel(layout);
			pmap.getModel().setProjectMap(pmap);
			p.addDocument(pmap);
			PluginServices.getMDIManager().addWindow(layout);


		}
	}
	/**
	 * Escribe el proyecto en XML.
	 *
	 * @param file Fichero.
	 * @param p Proyecto.
	 */
	public boolean writeProject(File file, Project p) {
		if( file.exists()){
			int resp = JOptionPane.showConfirmDialog(
					(Component) PluginServices.getMainFrame(),PluginServices.getText(this,"fichero_ya_existe_seguro_desea_guardarlo"),
					PluginServices.getText(this,"guardar"), JOptionPane.YES_NO_OPTION);
			if (resp != JOptionPane.YES_OPTION) {
				return false;
			}
		}
		// write it out as XML
		try {
            FileOutputStream fos = new FileOutputStream(file.getAbsolutePath());
            OutputStreamWriter writer = new OutputStreamWriter(fos, projectEncoding);
			Marshaller m = new Marshaller(writer);
			m.setEncoding(projectEncoding);
			p.setName(file.getName());
			// p.setPath(file.toString());
			p.setModificationDate(DateFormat.getDateInstance().format(new Date()));
			p.setModified(false);
			XMLEntity xml = p.getXMLEntity();
			xml.putProperty("followHeaderEncoding", true);
			m.marshal(xml.getXmlTag());
			PluginServices.getMainFrame().setTitle(file.getName());
			setPath(file.toString());

		} catch (Exception e) {
			NotificationManager.addError("Error guardando el proyecto", e);
			return false;
		}
		return true;
	}

	public Project readProject(String path) {
		BufferedReader reader =null;
		try {
		URL url=null;
			url = new URL(path);
			String encoding = XMLEncodingUtils.getEncoding(url.openStream());
			InputStream is = url.openStream();
			if (encoding!=null) {
				Project proj=null;
				try {
					reader = new BufferedReader(new InputStreamReader(is, encoding));
					proj=readProject(reader, true);
				} catch (UnsupportedEncodingException e) {
					reader = new BufferedReader(new InputStreamReader(is));
					try {
						Project p=readProject(reader, false);
					} catch (UnsupportedEncodingException e1) {
						JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this, e1.getLocalizedMessage()));
						return null;
					}
				}
				ProjectExtension.setPath(path);
				return proj;
			}
			else {
				reader = new BufferedReader(new InputStreamReader(is));
				Project p;
				try {
					p = readProject(reader, false);
				} catch (UnsupportedEncodingException e) {
					JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this, e.getLocalizedMessage()));
					return null;
				}
				ProjectExtension.setPath(path); //p.setPath(null);
				return p;
			}
		} catch (MalformedURLException e) {
			File file=new File(path);
			return readProject(file);
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Lee del XML el proyecto.<br><br>
	 *
	 * Reads the XML of the project.<br>
	 * It returns a project object holding all needed info that is not linked to the Project Dialog. <br>In case you want the project to be
	 * linked to the window you must set this object to the extension:<br>
	 *
	 * <b>Example:</b><br>
	 *
	 * ...<br>
	 * ...<br>
	 * Project p = ProjectExtension.readProject(projectFile);<br>
	 * ProjectExtension.setProject(p);
	 * ...<br>
	 * ...<br>
	 * @param file Fichero.
	 *
	 * @return Project
	 *
	 */
	public Project readProject(File file) {
		File xmlFile = new File(file.getAbsolutePath());
		try {
			String encoding = XMLEncodingUtils.getEncoding(new FileInputStream(xmlFile));
			InputStreamReader reader=null;
			if (encoding!=null) {
				try {
					reader = new InputStreamReader(new FileInputStream(xmlFile), encoding);
					return readProject(reader, true);
				} catch (UnsupportedEncodingException e) {
					reader = new InputStreamReader(new FileInputStream(xmlFile));
					try {
						return readProject(reader, false);
					} catch (UnsupportedEncodingException e1) {
						JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this, e1.getLocalizedMessage()));
						return null;
					}
				}
			}
			else {
				reader = new InputStreamReader(new FileInputStream(xmlFile));
				try {
					return readProject(reader, false);
				} catch (UnsupportedEncodingException e1) {
					JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this, e1.getLocalizedMessage()));
					return null;
				}
			}
		} catch (FileNotFoundException e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this, "fichero_incorrecto"));
			return null;
		}
	}

	/**
	 * Lee del XML el proyecto.<br><br>
	 *
	 * Reads the XML of the project.<br>
	 * It returns a project object holding all needed info that is not linked to the Project Dialog. <br>In case you want the project to be
	 * linked to the window you must set this object to the extension:<br>
	 *
	 * <b>Example:</b><br>
	 *
	 * ...<br>
	 * ...<br>
	 * Project p = ProjectExtension.readProject(projectFile);<br>
	 * ProjectExtension.setProject(p);
	 * ...<br>
	 * ...<br>
	 * @param reader File reader
	 * @param encodingFollowed Whether the encoded specified in the xml header was followed
	 * when creating the reader. If the property followHeaderEncoding is false or not set,
	 * then the encoding should have not been used when creating the reader, so it must be
	 * recreated.
	 *
	 * @return Project
	 * @throws UnsupportedEncodingException
	 *
	 */
	private Project readProject(Reader reader, boolean encodingFollowed) throws UnsupportedEncodingException {
		Project proj = null;

		try {
			XmlTag tag = (XmlTag) XmlTag.unmarshal(reader);
			XMLEntity xml=new XMLEntity(tag);
			String VERSION=xml.getStringProperty("VERSION");

			if (encodingFollowed) {
				if (xml.contains("followHeaderEncoding")) {
					boolean useEncoding = xml.getBooleanProperty("followHeaderEncoding");
					if (!useEncoding) {
						throw new UnsupportedEncodingException("the encoding specified in the xml header is not safe");
					}
				}
				else {
					// Old projects didn't contain followHeaderEncoding and they were
					// not correctly encoded. We throw an exception now, and we'll try
					// to reopen the project
					// using the default system encoding.
					throw new UnsupportedEncodingException("the encoding specified in the xml header is not safe");
				}
			}

			try {
				// if ((VERSION!=null) && (VERSION.equals("0.5") || VERSION.equals("0.4") || (VERSION.indexOf("GISPLANET") != -1))){
				if (VERSION != null) {
					proj = Project.createFromXML(xml);
				}else{
					proj = Project.createFromXML03(new XMLEntity(tag));
				}
				return proj;
			} catch (OpenException e){
				e.showError();
				//NotificationManager.addInfo("Al leer el proyecto", e);
			}
		}  catch (MarshalException e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this, "formato_incorrecto"));
			//NotificationManager.addError("Al leer el proyecto", e);
		} catch (ValidationException e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this, "formato_incorrecto"));
			//NotificationManager.addError("Al leer el proyecto", e);
		}
		return null;
	}

	/**
	 * Devuelve el proyecto.
	 *
	 * @return Proyecto.
	 */
	public Project getProject() {
		return p;
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#isEnabled()
	 */
	public boolean isEnabled() {
		return true;
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#isVisible()
	 */
	public boolean isVisible() {
		return true;
	}

	/**
	 * Sets the project
	 * @param p
	 */
	public void setProject(Project p){
		getProjectFrame().setProject(p);
		this.p=p;
	}

	//Finalizar y preguntar si se quiere guardar el proyecto o cancelar.
	public void terminate() {
		long t1,t2;
		t1 = System.currentTimeMillis();
		//if (getProject().hasChanged()) {
			int option = JOptionPane.showConfirmDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this,"guardar_cambios"),
					PluginServices.getText(this,"guardar_proyecto"),
					JOptionPane.YES_NO_OPTION);

			if (option == JOptionPane.YES_OPTION) {
				if (!guardar())
					terminate();
			}
		//}
		t2 = System.currentTimeMillis();
		System.err.println("ProjectExtension.terminate() " +  (t2-t1) + "milisecs");
	}

	private void registerDocuments() {
		ProjectViewFactory.register();
		ProjectTableFactory.register();
		ProjectMapFactory.register();
	}
	private void initializeDocumentActionsExtensionPoint() {
		ExtensionPoints extensionPoints = ExtensionPointsSingleton.getInstance();
		if (!extensionPoints.containsKey("DocumentActions_View")) {
			extensionPoints.put(
				new ExtensionPoint(
					"DocumentActions_View",
					"Context menu options of the view document list" +
						" in the project window "+
						"(register instances of " +
						"com.iver.cit.gvsig.project.AbstractDocumentContextMenuAction)"
				)
			);
		}
		if (!extensionPoints.containsKey("DocumentActions_Table")) {
			extensionPoints.put(
				new ExtensionPoint(
					"DocumentActions_Table",
					"Context menu options of the table document list" +
						" in the project window "+
						"(register instances of " +
						"com.iver.cit.gvsig.project.AbstractDocumentContextMenuAction)"
				)
			);
		}
		if (!extensionPoints.containsKey("DocumentActions_Map")) {
			extensionPoints.put(
				new ExtensionPoint(
					"DocumentActions_Map",
					"Context menu options of the map document list" +
						" in the project window "+
						"(register instances of " +
						"com.iver.cit.gvsig.project.AbstractDocumentContextMenuAction)"
				)
			);
		}
	}

	public static String getPath() {
		return projectPath;
	}

	public static void setPath(String path) {
		projectPath = path;
	}

	public IWindow getProjectWindow() {
		return getProjectFrame();
	}
}
