/*
 * Created on 01-jun-2004
 *
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.project.documents.view.legend.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Types;
import java.util.ArrayList;

import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import org.gvsig.gui.beans.swing.GridBagLayoutPanel;
import org.gvsig.gui.beans.swing.JButton;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.layerOperations.ClassifiableVectorial;
import com.iver.cit.gvsig.fmap.rendering.Legend;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.cit.gvsig.gui.panels.ColorChooserPanel;
import com.iver.cit.gvsig.gui.utils.FontChooser;

/**
 * @author fjp
 *
 */
public class FPanelLegendLabels extends JPanel implements ILegendPanel {

	private static final Font DEFAULT_FONT =
		  new Font("SansSerif", Font.PLAIN, 9);

	private VectorialLegend m_Renderer;
	private ClassifiableVectorial m_lyr;
	private FSymbol m_FSymbol;

	private ColorChooserPanel m_colorChooser = new ColorChooserPanel();
	private javax.swing.JCheckBox jCheckBox = null;
	private javax.swing.JComboBox jComboBoxTextField = null;
	private javax.swing.JComboBox jComboBoxHeightField = null;
	private javax.swing.JTextField jTextField = null;
	private javax.swing.JRadioButton jRadioButton = null;
	private javax.swing.JRadioButton jRadioButton1 = null;
	private FPreviewSymbol fPreviewSymbol = null;
	private javax.swing.JLabel jLabel3 = null;
	private JButton jButton = null;

	private Font labelFont = DEFAULT_FONT;
	private javax.swing.JLabel jLabel4 = null;
    private JComboBox jComboBoxRotationField = null;
    private JCheckBox jCheckBoxTextOnly = null;
	private GridBagLayoutPanel gridBagLayoutPanel = null;
	private JRadioButton rdBtnUseHeightField;
	private JRadioButton rdBtnFixedHeight;
	private final String NULL_COMBO_ITEM = "- " +PluginServices.getText(FPanelLegendLabels.class, "none") + " -";
	/**
	 * This is the default constructor
	 */
	public FPanelLegendLabels(JPanel parent) {
		super();
		initialize();
	}
	/**
	 * This method initializes this
	 *
	 * @return void
	 */
	private  void initialize() {
		this.setLayout(null);
		this.setSize(454, 308);
		this.add(getJCheckBox(), null);
		this.add(getJCheckBoxTextOnly(),null);
		this.add(getFPreviewSymbol(), null);
		this.add(getJLabel3(), null);
		this.add(getJButton(), null);
		ButtonGroup group = new ButtonGroup();
		group.add(getRdBtnUseHeightField());
		group.add(getRdBtnFixedHeight());
		ButtonGroup jButtonGroup = new ButtonGroup();
		jButtonGroup.add(getJRadioButton1());
		jButtonGroup.add(getJRadioButton());

		m_colorChooser.setAlpha(255);
		m_colorChooser.setBounds(336, 114, 99, 34);
		if (getJComboBoxTextField().getSelectedIndex()!=-1)
			m_colorChooser.setEnabled(true);
		else
			m_colorChooser.setEnabled(false);
		m_colorChooser.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateControls();
			}
		});
		this.add(getJLabel4(), null);
		this.add(m_colorChooser);

		this.add(getGridBagLayoutPanel(), null);
	}

	public void setLayer(FLayer lyr, Legend r)
	{
		m_lyr = (ClassifiableVectorial) lyr;
		m_Renderer = (VectorialLegend)r;

		fillFieldNames();
		getJComboBoxTextField().getModel().setSelectedItem(m_Renderer.getLabelField());
        getJComboBoxHeightField().getModel().setSelectedItem(m_Renderer.getLabelHeightField());
        getJComboBoxRotationField().getModel().setSelectedItem(m_Renderer.getLabelRotationField());

        if (r.getDefaultSymbol() instanceof FSymbol)
        	setFSymbol((FSymbol) r.getDefaultSymbol());

		if (m_Renderer.getLabelField() != null)
			getJCheckBox().setSelected(true);
		else
			getJCheckBox().setSelected(false);
	}

	public void setFSymbol(FSymbol sym)
	{
		if (sym == null)
		{
			m_FSymbol = new FSymbol(FShape.MULTI);
		}
		else
			m_FSymbol = sym;

		if (m_FSymbol.getFontColor()!=null){
			m_colorChooser.setColor(m_FSymbol.getFontColor());
		}
		fPreviewSymbol.setSymbol(m_FSymbol);

		getJRadioButton1().setSelected(!m_FSymbol.isFontSizeInPixels());
        getJCheckBoxTextOnly().setSelected(!m_FSymbol.isShapeVisible());
        double size = m_FSymbol.getFontSize();
        // Scale the value of the font size to the value of the height
        // and avoid too much decimal numbers
        size = ((int) (100*size / FConstant.FONT_HEIGHT_SCALE_FACTOR))/100;
		getJTextField().setText(String.valueOf(size));


	}

	private void updateControls() {
		getJComboBoxTextField().setEnabled(getJCheckBox().isSelected());
		if (getJComboBoxTextField().getSelectedIndex()!=-1){
			getJComboBoxHeightField().setEnabled(getJCheckBox().isSelected());
			getJComboBoxRotationField().setEnabled(getJCheckBox().isSelected());
		}
		getJRadioButton().setEnabled(getJCheckBox().isSelected());
		getJRadioButton1().setEnabled(getJCheckBox().isSelected());
		getRdBtnFixedHeight().setEnabled(getJCheckBox().isSelected());
		getRdBtnUseHeightField().setEnabled(getJCheckBox().isSelected());
		getJTextField().setEnabled(getJCheckBox().isSelected());
		getJCheckBoxTextOnly().setEnabled(getJCheckBox().isSelected());
		if (m_colorChooser != null)
			m_colorChooser.setEnabled(getJCheckBox().isSelected());
		getJButton().setEnabled(getJCheckBox().isSelected());
		if (getJCheckBox().isSelected())
		{
			FSymbol sym = m_FSymbol;
			sym.setFont(labelFont);
			sym.setFontColor(m_colorChooser.getColor());
			fPreviewSymbol.setSymbol(sym);
		}
	}

	public void updateValuesFromControls(VectorialLegend l) {
		if (getJCheckBox().isSelected())
		{
            m_FSymbol = (FSymbol) l.getDefaultSymbol();
            m_FSymbol.setFont(labelFont);
            String item = (String) getJComboBoxTextField().getSelectedItem();
            if (item == null || item.equals(NULL_COMBO_ITEM))
            	item = null;
			l.setLabelField(item);

			item = (String) getJComboBoxHeightField().getSelectedItem();
            if (item == null || item.equals(NULL_COMBO_ITEM))
            	item = null;
			l.setLabelHeightField(item);

			item = (String) getJComboBoxRotationField().getSelectedItem();
            if (item == null || item.equals(NULL_COMBO_ITEM))
            	item = null;
            l.setLabelRotationField(item);
			m_FSymbol.setFontColor(m_colorChooser.getColor());
			//m_FSymbol.setSymbolType(m_Renderer.getShapeType());
			String s=getJTextField().getText().replace(',','.');
			float fontSize = Float.parseFloat(s);
			fontSize *= FConstant.FONT_HEIGHT_SCALE_FACTOR;
			if (!getJRadioButton().isSelected())
			{
				// meters
				/*
				 * Java scales font size by defining the cell diagonal.
				 * To obtain the right height value we need to apply a
				 * factor to the diagonal to get a right value for the
				 * height.
				 */
				m_FSymbol.setFontSize(fontSize); // Para que se pueda crear correctamente al recuperar del proyecto
				m_FSymbol.setFontSizeInPixels(false);
			}
			else
			{
				// pixels
				m_FSymbol.setFont(labelFont);
				m_FSymbol.setFont(labelFont.deriveFont(fontSize));
				m_FSymbol.setFontSize(fontSize);
				m_FSymbol.setFontSizeInPixels(true);
			}
            m_FSymbol.setShapeVisible(true);
            if (getJCheckBoxTextOnly().isSelected())
                m_FSymbol.setShapeVisible(false);

			l.setDefaultSymbol(m_FSymbol);
            // System.out.println("Color punto 3 " + m_FSymbol.getColor().toString());
		}
		else
		{
			// Para que no se dibujen las etiquetas, el campo labelField del Renderer
			// ha de ser null
			l.setLabelField(null);
		}
	}

	private void fillFieldNames() {
		SelectableDataSource rs;
		try {
			// rs = ((ProjectExtension) PluginServices.getExtension(ProjectExtension.class)).getProject().getDataSourceByLayer((FLayer) m_lyr);
			rs = ((FLyrVect)m_lyr).getRecordset();
			String[] nomFields;
            ArrayList nomNumericFields = new ArrayList();

            // null value
            nomNumericFields.add(NULL_COMBO_ITEM);
			nomFields = new String[rs.getFieldCount()];
			for (int i = 0; i < rs.getFieldCount(); i++) {
				nomFields[i] = rs.getFieldName(i).trim();
                if ((rs.getFieldType(i) == Types.FLOAT)
                   || (rs.getFieldType(i) == Types.BIGINT)
                   || (rs.getFieldType(i) == Types.DOUBLE)
                   || (rs.getFieldType(i) == Types.INTEGER))
                    nomNumericFields.add(nomFields[i]);
			}

			DefaultComboBoxModel cM_labelField = new DefaultComboBoxModel(nomFields);
			getJComboBoxTextField().setModel(cM_labelField);
			DefaultComboBoxModel cM_HeightField = new DefaultComboBoxModel(nomNumericFields.toArray(new String[0]));
			getJComboBoxHeightField().setModel(cM_HeightField);
            DefaultComboBoxModel cM_RotationField = new DefaultComboBoxModel(nomNumericFields.toArray(new String[0]));
            getJComboBoxRotationField().setModel(cM_RotationField);

		} catch (DriverException e) {
			e.printStackTrace();
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method initializes jCheckBox
	 *
	 * @return javax.swing.JCheckBox
	 */
	private javax.swing.JCheckBox getJCheckBox() {
		if (jCheckBox == null) {
			jCheckBox = new javax.swing.JCheckBox();
			jCheckBox.setText(PluginServices.getText(this,"Habilitar_etiquetado"));
			jCheckBox.setBounds(2, 1, 152, 23);
			jCheckBox.addItemListener(new java.awt.event.ItemListener() {

				public void itemStateChanged(java.awt.event.ItemEvent e) {
					updateControls();

				}
			});

		}
		return jCheckBox;
	}

	/**
	 * This method initializes jComboBox
	 *
	 * @return javax.swing.JComboBox
	 */
	private javax.swing.JComboBox getJComboBoxTextField() {
		if (jComboBoxTextField == null) {
			jComboBoxTextField = new javax.swing.JComboBox();
			jComboBoxTextField.setEnabled(false);
			jComboBoxTextField.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					if (jComboBoxTextField.getSelectedIndex()!=-1){
						getJComboBoxHeightField().setEnabled(true);
						getJComboBoxRotationField().setEnabled(true);
					}else{
						getJComboBoxHeightField().setEnabled(false);
						getJComboBoxRotationField().setEnabled(false);
					}
				}
			});
		}
		return jComboBoxTextField;
	}

	/**
	 * This method initializes jComboBox1
	 *
	 * @return javax.swing.JComboBox
	 */
	private javax.swing.JComboBox getJComboBoxHeightField() {
		if (jComboBoxHeightField == null) {
			jComboBoxHeightField = new javax.swing.JComboBox();
			jComboBoxHeightField.setVisible(true);
			if (getJComboBoxTextField().getSelectedIndex()!=-1){
				jComboBoxHeightField.setEnabled(true);
			}else{
				jComboBoxHeightField.setEnabled(false);
			}

		}
		return jComboBoxHeightField;
	}

	/**
	 * This method initializes jTextField
	 *
	 * @return javax.swing.JTextField
	 */
	private javax.swing.JTextField getJTextField() {
		if (jTextField == null) {
			jTextField = new javax.swing.JTextField();
			jTextField.setPreferredSize(new Dimension(76, 21));
			jTextField.setBounds(26, 32, 76, 21);
			if (getJComboBoxTextField().getSelectedIndex()!=-1){
				jTextField.setEnabled(true);
			}else{
				jTextField.setEnabled(false);
			}
			jTextField.setEnabled(false);
			jTextField.setText("10");
		}
		return jTextField;
	}

	/**
	 * This method initializes jRadioButton
	 *
	 * @return javax.swing.JRadioButton
	 */
	private javax.swing.JRadioButton getJRadioButton() {
		if (jRadioButton == null) {
			jRadioButton = new javax.swing.JRadioButton();
			jRadioButton.setText(PluginServices.getText(this,"En_pixels"));
			jRadioButton.setBounds(145, 44, 100, 23);
			if (getJComboBoxTextField().getSelectedIndex()!=-1){
				jRadioButton.setEnabled(true);
			}else{
				jRadioButton.setEnabled(false);
			}
			jRadioButton.setEnabled(false);
			jRadioButton.setSelected(true);
		}
		return jRadioButton;
	}

	/**
	 * This method initializes jRadioButton1
	 *
	 * @return javax.swing.JRadioButton
	 */
	private javax.swing.JRadioButton getJRadioButton1() {
		if (jRadioButton1 == null) {
			jRadioButton1 = new javax.swing.JRadioButton();
			jRadioButton1.setText(PluginServices.getText(this,"En_metros"));
			jRadioButton1.setBounds(145, 16, 100, 23);
			if (getJComboBoxTextField().getSelectedIndex()!=-1){
				jRadioButton1.setEnabled(true);
			}else{
				jRadioButton1.setEnabled(false);
			}
			jRadioButton1.setEnabled(false);
		}
		return jRadioButton1;
	}

	/**
	 * This method initializes fPreviewSymbol
	 *
	 * @return com.iver.cit.opensig.gui.Panels.FPreviewSymbol
	 */
	public FPreviewSymbol getFPreviewSymbol() {
		if (fPreviewSymbol == null) {
			fPreviewSymbol = new FPreviewSymbol();
			fPreviewSymbol.setBounds(311, 53, 134, 77);
		}
		return fPreviewSymbol;
	}

	/**
	 * This method initializes jLabel3
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabel3() {
		if (jLabel3 == null) {
			jLabel3 = new javax.swing.JLabel();
			jLabel3.setBounds(336, 27, 100, 18);
			jLabel3.setText(PluginServices.getText(this,"Previsualizacion"));
			jLabel3.setVisible(false);
		}
		return jLabel3;
	}

	/**
	 * This method initializes jButton
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setBounds(338, 53, 111, 29);
			if (getJComboBoxTextField().getSelectedIndex()!=-1)
				jButton.setEnabled(true);
			else
				jButton.setEnabled(false);
			jButton.setText(PluginServices.getText(this,"fuente")+"...");
			jButton.addActionListener(new java.awt.event.ActionListener() {

				public void actionPerformed(java.awt.event.ActionEvent e) {
					//JButton origen = (JButton) e.getSource();
					Font newFont;

					newFont = FontChooser.showDialog("Choose Font", labelFont);

			        if (newFont == null) {
			            return;
			        }

			        labelFont = newFont;
			        updateControls();


				}
			});

		}
		return jButton;
	}

	/**
	 * This method initializes jLabel4
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabel4() {
		if (jLabel4 == null) {
			jLabel4 = new javax.swing.JLabel();
			jLabel4.setBounds(335, 87, 115, 23);
			jLabel4.setText(PluginServices.getText(this,"color_texto"));
		}
		return jLabel4;
	}

	public Legend getLegend() {
		updateValuesFromControls(m_Renderer);
		return m_Renderer;
	}
    /**
     * This method initializes jComboBox1
     *
     * @return javax.swing.JComboBox
     */
    private JComboBox getJComboBoxRotationField() {
    	if (jComboBoxRotationField == null) {
    		jComboBoxRotationField = new JComboBox();
    		if (getJComboBoxTextField().getSelectedIndex()!=-1)
				jComboBoxRotationField.setEnabled(true);
			else
				jComboBoxRotationField.setEnabled(false);
    	}
    	return jComboBoxRotationField;
    }

    /**
     * This method initializes jCheckBox1
     *
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getJCheckBoxTextOnly() {
    	if (jCheckBoxTextOnly == null) {
    		jCheckBoxTextOnly = new JCheckBox();
    		jCheckBoxTextOnly.setBounds(315, 171, 200, 22);
    		if (getJComboBoxTextField().getSelectedIndex()!=-1){
				jCheckBoxTextOnly.setEnabled(true);
			}else{
				jCheckBoxTextOnly.setEnabled(false);
			}
    		jCheckBoxTextOnly.setText(PluginServices.getText(this, "draw_text_only"));
    	}
    	return jCheckBoxTextOnly;
    }
	/**
	 * This method initializes gridBagLayoutPanel
	 *
	 * @return org.gvsig.gui.beans.swing.GridBagLayoutPanel
	 */
	private GridBagLayoutPanel getGridBagLayoutPanel() {
		if (gridBagLayoutPanel == null) {
			gridBagLayoutPanel = new GridBagLayoutPanel();
			gridBagLayoutPanel.setBounds(new java.awt.Rectangle(14,15,263,274));
			gridBagLayoutPanel.addComponent(new JLabel(PluginServices.getText(this,"Campo_de_etiquetado")));
			gridBagLayoutPanel.addComponent("", getJComboBoxTextField());
			gridBagLayoutPanel.addComponent(new JLabel(PluginServices.getText(this, "rotation_field")));
			gridBagLayoutPanel.addComponent("", getJComboBoxRotationField());

			gridBagLayoutPanel.addComponent(getRdBtnUseHeightField());
			gridBagLayoutPanel.addComponent("", getJComboBoxHeightField());

			JPanel aux = new JPanel(new BorderLayout(0, 0));
			JPanel aux1 = new JPanel(new FlowLayout(FlowLayout.LEADING));
			aux1.add(getJTextField(), BorderLayout.CENTER);
			JPanel aux2 = new JPanel(new GridLayout(2,1));
			aux2.add(getJRadioButton());
			aux2.add(getJRadioButton1());
			aux.add(aux1, BorderLayout.WEST);
			aux.add(aux2, BorderLayout.CENTER);
			gridBagLayoutPanel.addComponent(getRdBtnFixedHeight());
			gridBagLayoutPanel.addComponent("", aux);
			//gridBagLayoutPanel.addComponent("", getJCheckBoxTextOnly());

		}
		return gridBagLayoutPanel;
	}

	private JRadioButton getRdBtnUseHeightField() {
		if (rdBtnUseHeightField == null) {
			rdBtnUseHeightField = new JRadioButton(PluginServices.getText(this,"campo_altura_texto"));
			rdBtnUseHeightField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					boolean b = rdBtnUseHeightField.isSelected();
					getJTextField().setEnabled(!b);
					getJComboBoxHeightField().setEnabled(b);
					getJRadioButton().setEnabled(!b);
					getJRadioButton1().setEnabled(!b);
				}
			});
		}
		return rdBtnUseHeightField;
	}

	private JRadioButton getRdBtnFixedHeight() {
		if (rdBtnFixedHeight == null) {
			rdBtnFixedHeight = new JRadioButton(PluginServices.getText(this,"Altura_fija_de_texto")+":");
			rdBtnFixedHeight.setSelected(true);
			rdBtnFixedHeight.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					boolean b = rdBtnFixedHeight.isSelected();
					getJTextField().setEnabled(b);
					getJComboBoxHeightField().setEnabled(!b);
					getJRadioButton().setEnabled(b);
					getJRadioButton1().setEnabled(b);
					getJComboBoxHeightField().setSelectedItem(NULL_COMBO_ITEM);
				}

			});
		}
		return rdBtnFixedHeight;
	}

	public void repaint() {
		updateControls();
		super.repaint();
	}
}