/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */

package com.iver.cit.gvsig.project.documents.view.toc.gui;

import org.cresques.filter.RasterFilter;
import org.cresques.filter.RasterFilterStackManager;
import org.cresques.filter.enhancement.BrightnessContrastStackManager;
import org.cresques.ui.BrightnessContrast.EnhancedBrightnessContrastPanel;
import org.cresques.ui.filter.FilterRasterDialogPanel;

import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.gui.panels.PropertiesRasterDialog;

/**
 * Entrada al menu TOC que contiene la parter del listener que corresponde a los 
 * filtros de brillo y contraste. El metodo actionPerformed de esta clase sera llamado
 * por FLyrRasterAdjustPropertiesTocMenuEntry. Para ello, tiene que registrarse usando
 * el metodo register.
 * 
 * @author Miguel ngel Querol Carratal  <querol_mig@gva.es>
 *
 */

public class BrightnessContrastTocMenuEntry{

	EnhancedBrightnessContrastPanel panel = null;
	BrightnessContrastStackManager bcManager = null;
	
	/**
	 * Constructor. Se registra a si mismo en las propiedades de raster del TOC.
	 * @param menu
	 */
	public BrightnessContrastTocMenuEntry (){
		
	}
	
	/**
	 * Modificacin del estado de los controles del panel de brillo y contraste
	 */
	public void actionPerformed(RasterFilterStackManager stackManager, PropertiesRasterDialog propsDialog, FLyrRaster fLayer) {
		EnhancedBrightnessContrastPanel bcPanel = (EnhancedBrightnessContrastPanel)((FilterRasterDialogPanel)propsDialog.getContentPane()).getPanelByClassName("EnhancedBrightnessContrastPanel"); 
		//bcPanel.setPropertiesDialog(propsDialog);
		
		if(stackManager.isActive(stackManager.getTypeFilter("brightness"))){
			bcPanel.getCBrightC().setSelected(true);
			bcPanel.setBCControlEnabled(true);
			
			RasterFilter bright = stackManager.getFilter("brightness");
			
			if(bright.getParam("incrBrillo") != null){
				int incr = ((Integer)bright.getParam("incrBrillo")).intValue();
				bcPanel.getLabelSliderText().setSliderValue(incr);
				bcPanel.getLabelSliderText().setTextValue(String.valueOf(incr));
			}
		}
		
		if(stackManager.isActive(stackManager.getTypeFilter("contrast"))){
			bcPanel.getCBrightC().setSelected(true);
			bcPanel.setBCControlEnabled(true);
			
			RasterFilter cont = stackManager.getFilter("contrast");
			
			if(cont.getParam("incrContraste") != null){
				int incr = ((Integer)cont.getParam("incrContraste")).intValue();
				bcPanel.getLabelSliderText1().setSliderValue(incr);
				bcPanel.getLabelSliderText1().setTextValue(String.valueOf(incr));
			}
		}
		
		if(stackManager.isActive(stackManager.getTypeFilter("enhanced"))){
			bcPanel.getCEnhanced().setSelected(true);
			bcPanel.setEControlEnabled(true);
			
			RasterFilter enhan = stackManager.getFilter("enhanced");
			RasterFilter tail = stackManager.getFilter("tail");
			
			
				boolean rem = ((Boolean)enhan.getParam("remove")).booleanValue();
				bcPanel.getJCheckBox().setSelected(rem);
			
			if(stackManager.isActive(stackManager.getTypeFilter("tail"))){
				double percent = (stackManager.getStackStats().tailPercent)*100;
				bcPanel.getCheckSliderText().setTextValue(String.valueOf(percent));
				bcPanel.getCheckSliderText().setSliderValue((int)percent);
				bcPanel.getCheckSliderText().setSelected(true);
				bcPanel.getCheckSliderText().setControlEnabled(true);
				boolean remove = ((Boolean)tail.getParam("remove")).booleanValue();
				if((remove == true) || (rem == true))
					bcPanel.getJCheckBox().setSelected(true);
				else 
					bcPanel.getJCheckBox().setSelected(false);
				
			}
				
			
		}
	}
	
}
