package es.gva.cit.gvsig.catalogClient.loaders;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.util.HashMap;
import java.util.Map;

import org.exolab.castor.xml.ValidationException;
import org.gvsig.i18n.Messages;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.drivers.UnsupportedVersionException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.project.documents.view.gui.View;
import com.iver.utiles.extensionPoints.ExtensionPoint;
import com.iver.utiles.extensionPoints.ExtensionPointsSingleton;

import es.gva.cit.catalogClient.schemas.discoverer.Resource;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: ARCIMSLayerLoader.java,v 1.1.2.2 2006/11/15 00:08:16 jjdelcerro Exp $
 * $Log: ARCIMSLayerLoader.java,v $
 * Revision 1.1.2.2  2006/11/15 00:08:16  jjdelcerro
 * *** empty log message ***
 *
 * Revision 1.2  2006/10/02 08:29:06  jorpiell
 * Modificados los cambios del Branch 10 al head
 *
 * Revision 1.1.2.1  2006/09/20 12:01:36  jorpiell
 * Se ha cambiado la versin de la jzkit, se ha incorporado la funcionalidad de cargar arcims
 *
 * Revision 1.1  2006/09/20 11:23:50  jorpiell
 * Se ha cambiado la versin de la librera jzkit de la 1 a la 2.0. Adems se ha modificado lo del document
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class ARCIMSLayerLoader extends LayerLoader {
	
	public ARCIMSLayerLoader(Resource resource) {
		super(resource);		
	}
	
	/**
	 * This function loads a ArcIms resource 
	 * @param host
	 * URL where the server is located
	 * @param sLayer
	 * Layer name
	 * @param crs
	 * Coordenates System
	 * @throws LayerLoaderException 
	 */
	
	public void loadLayer() throws LayerLoaderException {
		String host = getResource().getLinkage();
		String sLayer = getResource().getName();
		String protocol = getResource().getProtocol();
		View activeView = 
			(View) PluginServices.getMDIManager().getActiveWindow();
		String srs = activeView.getProjection().getAbrev();
		FLayer flayer = null;
				
		Map args = new HashMap();
		args.put("host",host);
		args.put("service_name",sLayer);
		args.put("layer_name",sLayer);
		args.put("srs",srs);
		
		if (protocol.toUpperCase().indexOf(Resource.ARCIMS_IMAGE) >= 0){
			flayer = createArcImsImageLayer(args);
		}else if (protocol.toUpperCase().indexOf(Resource.ARCIMS_VECTORIAL) >= 0){
			flayer = createArcImsVectorialLayer(args);
		}
		
		try {
			addLayerToView(flayer);
		} catch (Exception e) {
			throw new LayerLoaderException(e.getMessage(),getWindowMessage());
		}   		    
		
	}	

	private FLayer createArcImsImageLayer(Map args) throws  LayerLoaderException{
		ExtensionPoint extensionPoint = (ExtensionPoint)ExtensionPointsSingleton.getInstance().get("CatalogLayers");

		try {
			return (FLayer)extensionPoint.create("arcims_raster", args  );
		} catch(Exception e) {
			throw new LayerLoaderException(getErrorMessage(),getWindowMessage());
		}
	}
	
	private FLayer createArcImsVectorialLayer(Map args) throws  LayerLoaderException{
		ExtensionPoint extensionPoint = (ExtensionPoint)ExtensionPointsSingleton.getInstance().get("CatalogLayers");
		
		try {			
			return (FLayer)extensionPoint.create("arcims_vectorial", args);
		} catch(Exception e) {
			throw new LayerLoaderException(getErrorMessage(),getWindowMessage());
		}
	}
	
	/*
	 *  (non-Javadoc)
	 * @see es.gva.cit.gvsig.catalogClient.loaders.LayerLoader#getErrorMessage()
	 */
	protected String getErrorMessage() {
		return Messages.getText("arcims_server_error") + ".\n" +
		Messages.getText("server") + ": " + 
		getResource().getLinkage() + "\n" +
		Messages.getText("layer") + ": " +
		getResource().getName();		
	}
	
	/*
	 *  (non-Javadoc)
	 * @see es.gva.cit.gvsig.catalogClient.loaders.LayerLoader#getWindowMessage()
	 */
	protected String getWindowMessage() {
		return Messages.getText("arcims_load");
	}	
}

