/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.Dimension;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import com.iver.cit.gvsig.fmap.ViewPort;

/**
 * Punto de la capa que representa una coordenada en pixeles de la imagen a
 * georreferenciar, una coordenada de la vista, el viewPort de las mini imagenes
 * ,el centro de estas y el flag activo que dice si el punto tiene que ser tomado en
 * cuenta o no.
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class GeoPoint{
	public Point2D 	pixelPoint = null;
	public Point2D 	mapPoint = null;
	public ViewPort leftViewPort =  null, rightViewPort = null;
	public Point2D  leftCenterPoint = null, rightCenterPoint = null;
	public double zoomLeft = 1, zoomRight = 1;
	public boolean 	active = true;

	/**
	 * Constructor
	 */
	public GeoPoint(){}

	/**
	 * Constructor
	 * @param p pixelPoint
	 * @param m mapPoint
	 */
	public GeoPoint(Point2D p, Point2D m){
		this.pixelPoint = p;
		this.mapPoint = m;
	}

	/**
	 * Hace una copia de la instancia del punto.
	 * @return GeoPoint
	 */
	public GeoPoint cloneGeoPoint(){
		Point2D pixel = null;
		Point2D map = null;
		if(pixelPoint != null)
			pixel = (Point2D)pixelPoint.clone();

		if(mapPoint != null)
			map = (Point2D)mapPoint.clone();

		GeoPoint gp = new GeoPoint(pixel, map);

		if(leftViewPort != null){
			gp.leftViewPort = new ViewPort(leftViewPort.getProjection());
			gp.leftViewPort.setExtent((Rectangle2D)leftViewPort.getExtent().clone());
			gp.leftViewPort.setImageSize((Dimension)leftViewPort.getImageSize().clone());
			gp.leftViewPort.refreshExtent();
		}

		if(rightViewPort != null){
			gp.rightViewPort = new ViewPort(rightViewPort.getProjection());
			gp.rightViewPort.setExtent((Rectangle2D)rightViewPort.getExtent().clone());
			gp.rightViewPort.setImageSize((Dimension)rightViewPort.getImageSize().clone());
			gp.rightViewPort.refreshExtent();
		}

		if(leftCenterPoint != null)
			gp.leftCenterPoint = (Point2D)this.leftCenterPoint.clone();
		if(rightCenterPoint != null)
			gp.rightCenterPoint = (Point2D)this.rightCenterPoint.clone();
		gp.active = this.active;
		gp.zoomLeft = this.zoomLeft;
		gp.zoomRight = this.zoomRight;
		return gp;
	}

	/**
	 * Muestra por consola algunos valores de un geopunto
	 */
	public void show(){
		System.out.println("********GeoPoint**********");
		System.out.println("Pixel: "+pixelPoint);
		System.out.println("Map: "+mapPoint);
		System.out.println("Left Zoom Center: "+leftCenterPoint);
		System.out.println("Right Zoom Center: "+rightCenterPoint);
		System.out.println("Active: "+active);
		System.out.println("ZoomLeft: "+zoomLeft);
		System.out.println("ZoomRight: "+zoomRight);
		System.out.println("******End GeoPoint********");
	}
}