/*
 * Created on 17-feb-2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.rasterTools.saveRaster.ui.properties;

import java.awt.Dimension;
import java.awt.geom.Rectangle2D;

import org.cresques.cts.IProjection;
import org.cresques.io.GeoRasterWriter;
import org.gvsig.rasterTools.saveRaster.operations.SaveRasterThread;
import org.gvsig.rasterTools.saveRaster.ui.listener.ProgressBarListener;
import org.gvsig.rasterTools.saveRaster.ui.main.SaveRasterDefaultPanel;
import org.gvsig.rasterTools.saveRaster.util.DriverProperties;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.layers.FLayers;

/**
 * @author Nacho Brodin <brodin_ign@gva.es>
 */
public class ProgressSaveRasterDialog extends ProgressSaveRasterPanel implements IWindow{
	private ViewPort 					vp = null;
	private SaveRasterThread			threadSave = null;
	private	ProgressBarListener			progressBarListener = null;
	private SaveRasterDefaultPanel 		parent = null;
	
	/**
	 * Contructor 
	 * @param mapCtrl			MapControl
	 * @param layers			Capas a rasterizar
	 * @param currentProjection	Proyeccin 
	 * @param dimension			Dimensin del marco en pixeles
	 * @param ext				Extend de las capas a rasterizar
	 */
	public ProgressSaveRasterDialog(	MapControl mapCtrl, 
										FLayers layers, 
										IProjection currentProjection,
										Dimension dimension, 
										Rectangle2D ext,
										SaveRasterDefaultPanel parent){
		super();
	
		vp = new ViewPort(currentProjection);
		vp.setBackColor(mapCtrl.getViewPort().getBackColor());
		vp.setImageSize(dimension);
		vp.setExtent(ext);
		this.parent = parent;
		 
		threadSave = new SaveRasterThread(mapCtrl, layers, dimension, vp, this);
		progressBarListener = new ProgressBarListener(threadSave, this);
		this.setTranslation();
	}
	
	/**
	 * Asigna los textos a los paneles en el idioma seleccionado
	 */	
	private void setTranslation(){
		this.getLabelWait().setText(PluginServices.getText(this,"espere")+",...");
	}
	
	/**
	 * Asigna el objeto que contiene las propiedades
	 * @param driverProps
	 */
	public void setProps(DriverProperties driverProps){
		if(threadSave != null)
			threadSave.setProps(driverProps);
	}
	
	/**
	 * Asigna el nombre del fichero
	 * @param fName	Nombre del fichero
	 */
	public void setFName(String fName){
		if(threadSave != null)
			threadSave.setFName(fName);
	}
	
	/**
	 * Asigna el GeoRasterWriter que se usar para salvar
	 * @param writer
	 */
	public void setWriter(GeoRasterWriter writer){
		if(threadSave != null)
			threadSave.setWriter(writer);
	}
	
	/**
	 * Asigna el tamao de bloque que se lee de una sola vez desde el origen
	 * Esto influye en la velocidad de escritura y depende de la memoria que 
	 * tengamos disponible. Si disponemos de poca corremos el riesgo de que al
	 * asignar un bloque muy grande nos de un error
	 * @param sizeBlock Tamao de bloque
	 */
	public void setBlockSize(int sizeBlock){
		if(threadSave != null)
			threadSave.setBlockSize(sizeBlock);
	}
	
	/**
	 * Asigna la compresin si el formato dispone de este parmetro
	 * @param compresion Nivel de compresin
	 */
	public void setCompresion(int compresion){
		if(threadSave != null)
			threadSave.setCompresion(compresion);
	}
	
	/**
	 * Muestra la barra de incremento
	 */
	public void showWindow(){
		threadSave.start();
		PluginServices.getMDIManager().addWindow(this);
	}
		
	/**
	 * @see com.iver.mdiApp.ui.MDIManager.IWindow#getWindowInfo()
	 */
	public WindowInfo getWindowInfo() {
		WindowInfo m_viewinfo=new WindowInfo(WindowInfo.MODALDIALOG);
    		m_viewinfo.setTitle(PluginServices.getText(this, "tarea_en_progreso"));
		return m_viewinfo;
	}

	/**
	 * Obtiene la ventana padre de esta.
	 * @return SaveRasterDefaultPanel
	 */
	public SaveRasterDefaultPanel getParentWindow() {
		return parent;
	}
}


