/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.px.dxf;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;

import org.cresques.geo.ViewPortData;

import org.cresques.io.DxfGroup;

import org.cresques.px.Extent;

import java.awt.Graphics2D;
import java.awt.geom.Point2D;


/**
 * Entidad ATTRIB de un fichero DXF.
 * @author jmorell
 */
public class DxfAttrib extends DxfEntity {
    private Point2D pt;
    
    /**
     * Constructor de DxfAttrib.
     * @param proj, proyeccin cartogrfica en la que se encuentra el DxfAttrib.
     * @param layer, capa del DXF en la que se encuentra el DxfAttrib.
     */
    public DxfAttrib(IProjection proj, DxfLayer layer) {
        super(proj, layer);
        extent = new Extent();
        pt = new Point2D.Double();
    }
    
    /**
     * Establece el punto de insercin del DxfAttrib.
     * @param pt, punto de insercin.
     */
    public void setPt(Point2D pt) {
        this.pt = pt;
        extent.add(pt);
    }
    
    /**
     * Permite reproyectar un DxfAttrib dado un conjunto de coordenadas de transformacin.
     * @param rp, coordenadas de transformacin.
     */
    public void reProject(ICoordTrans rp) {
        Point2D savePt = pt;

        pt = new Point2D.Double();
        extent = new Extent();

        Point2D ptDest = rp.getPDest().createPoint(0.0, 0.0);

        if (savePt == null) {
            ptDest = null;
        } else {
            ptDest = rp.convert((Point2D) savePt, ptDest);
            extent.add(ptDest);
        }

        pt = ptDest;
        setProjection(rp.getPDest());
    }
    
    /**
     * Permite dibujar un DxfAttrib.
     */
    public void draw(Graphics2D g, ViewPortData vp) {
        //AffineTransform msave=g.getTransform();
        //g.setTransform(vp.mat);
        if (dxfColor == AcadColor.BYLAYER) {
            g.setColor(layer.getColor());
        } else {
            g.setColor(AcadColor.getColor(dxfColor));
        }

        Point2D ptT = new Point2D.Double(0, 0);
        vp.mat.transform(ptT, ptT);
        ptT.setLocation(pt.getX(), pt.getY());
        vp.mat.transform(ptT, ptT);
        g.drawLine((int) ptT.getX(), (int) ptT.getY(), (int) ptT.getX(),
                   (int) ptT.getY());

        //g.setTransform(msave);
    }

    /**
     * 050406, jmorell: Implementacin parcial. Est para DxfPoint. Falta hacerlo
     * para un DxfAttrib ...
     */
    public String toDxfString() {
        StringBuffer sb = null;
        sb = new StringBuffer(DxfGroup.toString(0, "POINT"));
        sb.append(DxfGroup.toString(5, getHandle()));
        sb.append(DxfGroup.toString(100, "AcDbEntity"));
        sb.append(DxfGroup.toString(8, layer.getName()));
        sb.append(DxfGroup.toString(62, dxfColor));
        sb.append(DxfGroup.toString(100, "AcDbPoint"));
        sb.append(DxfGroup.toString(10, pt.getX(), 6));
        sb.append(DxfGroup.toString(20, pt.getY(), 6));
        sb.append(DxfGroup.toString(30, 0.0, 6));

        return sb.toString();
    }

    /**
     * @return Returns the pt.
     */
    public Point2D getPt() {
        return pt;
    }
}
