/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.cmd;

import org.cresques.px.gml.Feature;
import org.cresques.px.gml.LineString;

import org.cresques.ui.CQCursor;
import org.cresques.ui.CQMapCanvas;

import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

import java.util.Vector;


/**
 * Comando pline.
 * Aade al canvas la capacidad de crear polilineas.
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
public class CmdPLine extends Cmd {
    private Point2D ptIni = null;
    private Vector pts = new Vector();
    private Feature pLine = null;

    /**
     * Construye un nuevo CmdPLine para el Canvas.
     * @param canvas
     */
    public CmdPLine(CQMapCanvas canvas) {
        super(canvas);
        eventsWanted = LEFT | RIGHT | PRESS | RELEASE;
        cursor = CQCursor.getCursor(CQCursor.SELECT_CURSOR);
    }

    /**
     * Recibe los eventos del ratn.
     */
    public void cmd(Point2D pt, int bt, int mouseEvent) {
        if (mouseEvent == Cmd.RELEASE) {
            if (bt == MouseEvent.BUTTON1) {
                pts.add(pt);
                pline();
            } else {
                pts = new Vector();
            }
        }
    }

    /**
     * Aade el punto actual a la PLine. La crea si no existe.
     * @param ptIni Punto inicial.
     * @param ptFin Punto final.
     */
    private void pline() {
        if (pts.size() > 2) {
            pLine.getGeometry().add((Point2D) pts.lastElement());
        } else if (pts.size() == 2) {
            pLine = new Feature();
            pLine.setGeometry(new LineString());
            pLine.getGeometry().add((Point2D) pts.get(0));
            pLine.getGeometry().add((Point2D) pts.get(1));
            canvas.getApp().getCurrentLayer().add(pLine);
        }

        canvas.repaint();
    }
}
