/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.drivers;

import org.cresques.cts.IProjection;

import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;

import java.io.IOException;


/**
 * Interfaz de Driver de raster georeferenciado.
 *
 * @author $author$
 */
public interface GeorreferencedRasterDriver extends RasterDriver {
	/**
	 * Inicializa el raster con la proyeccin que se le pasa.
	 *
	 * @param proj
	 *
	 * @throws IOException
	 */
	void initialize(IProjection proj) throws IOException;

	/**
	 * Obtiene la proyeccin en la que est el raster
	 *
	 * @return DOCUMENT ME!
	 */
	IProjection getProjection();

	/**
	 * Asigna la proyeccin en la que est el raster
	 *
	 * @param proj DOCUMENT ME!
	 */
	void setProjection(IProjection proj);

	/**
	 * Obtiene la extensin en coordenadas del mundo real que ocupa este raster
	 *
	 * @return DOCUMENT ME!
	 */
	Rectangle2D getFullExtent();
	
	/**
	 * Obtiene el flag que dice si la imagen est o no georreferenciada
	 * @return true si est georreferenciada y false si no lo est.
	 */
	public boolean isGeoreferenced();
	
	/**
	 * Asigna una transformacin al geoRasterFile para la transformar la 
	 * asignacin de setView. Del mismo modo asigna los extents a PxRaster para
	 * transformar la visualizacin.
	 * @param t Matriz de transformacin
	 */
	public void setAffineTransform(AffineTransform t);
	/**
	 * Obtiene la matriz de transformacin
	 * @return Matriz de transformacin
	 */
	public AffineTransform getAffineTransform();
}
