/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import com.iver.cit.gvsig.fmap.rendering.LegendChangedEvent;

import java.util.ArrayList;
import java.util.Iterator;


/**
 * Clase que gestiona el aadir, borrar y ejecutar los listeners de cambio de
 * estilo de las capas.
 *
 * @author Vicente Caballero Navarro
 */
public class LayerChangeSupport {
	private ArrayList listeners = new ArrayList();

	/**
	 * Aade un LegendListener al ArrayList de Listener.
	 *
	 * @param listener LegendListener.
	 */
	public void addLayerListener(LegendListener listener) {
		listeners.add(listener);
	}

	/**
	 * Borra un LegendListener del ArrayList.
	 *
	 * @param listener LegendListener.
	 */
	public void removeLayerListener(LegendListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Llamada al mtodo legendChanged de los listener.
	 *
	 * @param e LegendChangeEvent
	 */
	public void callLegendChanged(LegendChangedEvent e) {
		for (Iterator iter = listeners.iterator(); iter.hasNext();) {
			LegendListener listener = (LegendListener) iter.next();

			listener.legendChanged(e);
		}
	}
}
