/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers.layerOperations;

import java.awt.geom.Rectangle2D;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;



/**
 * Interfaz a implementar por las capas con datos vectoriales
 */
public interface VectorialData {
	/**
	 * Realiza un proceso sobre las geometras de la capa indicadas por el
	 * bitset que se pasa como parmetro
	 *
	 * @param visitor Visitor que se recibir las visitas de las geometras
	 * @param subset BitSet indicando los ndices de las geometras que
	 * 		  visitarn
	 *
	 * @throws DriverException
	 * @throws VisitException
	 */
	public void process(FeatureVisitor visitor, FBitSet subset)
		throws DriverException, VisitException;
	
	
	/**
	 * Realiza un proceso sobre las geometras de la capa que intersecten
	 * el rectangulo que se pasa como parametro
	 *
	 * @param visitor Visitor que se recibir las visitas de las geometras
	 * @param rectangle Indica el rectangulo que contendra a las geometrias
	 * que se procesaran
	 *
	 * @throws DriverException
	 * @throws VisitException
	 */
	public void process(FeatureVisitor visitor, Rectangle2D rect)
		throws DriverException, VisitException;

	/**
	 * Realiza un proceso sobre las geometras de la capa indicadas por el
	 * bitset que se pasa como parmetro
	 *
	 * @param visitor Visitor que se recibir las visitas de las geometras
	 *
	 * @throws DriverException
	 * @throws VisitException
	 */
	public void process(FeatureVisitor visitor)
		throws DriverException, VisitException;
}
