package com.hardcode.gdbms.engine.data.db;

import com.hardcode.gdbms.engine.data.driver.DBTransactionalDriver;
import com.hardcode.gdbms.engine.data.driver.DriverException;

import java.sql.SQLException;


/**
 * DataWare for the DataSourceFactory.DATA_WARE_COHERENT_ROW_ORDER mode
 * that keeps the row order during edition with the drawback of 
 * being more slow that the one that implements the 
 * DataSourceFactory.DATA_WARE_DIRECT_MODE mode
 *
 * @author Fernando Gonzlez Corts
 */
public class CoherentRowOrderDataWare extends FakeTransactionDataWare {
    /**
     * Gets the underlaying driver
     *
     * @return DBTransactionalDriver
     */
    private DBTransactionalDriver getTransacctionalDriver() {
        return (DBTransactionalDriver) driver;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.edition.DataWare#beginTrans()
     */
    public void beginTrans() throws DriverException {
        super.beginTrans();

        try {
            getTransacctionalDriver().beginTrans(con);
        } catch (SQLException e) {
            throw new DriverException(e);
        }
    }

    /**
     * @see com.hardcode.gdbms.engine.data.edition.DataWare#commitTrans()
     */
    public void commitTrans() throws DriverException {

        try {
            getTransacctionalDriver().commitTrans(con);
            super.commitTrans();
        } catch (SQLException e) {
            throw new DriverException(e);
        }
    }

    /**
     * @see com.hardcode.gdbms.engine.data.edition.DataWare#rollBackTrans()
     */
    public void rollBackTrans() throws DriverException {

        try {
            getTransacctionalDriver().rollBackTrans(con);
            super.rollBackTrans();
        } catch (SQLException e) {
            throw new DriverException(e);
        }
    }
}
