package com.hardcode.gdbms.engine.data.edition;

import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueCollection;


/**
 * Interface that provide transactional editing capabilities
 *
 * @author Fernando Gonzlez Corts
 */
public interface DataWare extends DataSource {
    /**
     * Deletes the ith row of the DataSource
     *
     * @param rowId index of the row to be deleted
     *
     * @throws DriverException if the row could not be deleted
     */
    public void deleteRow(long rowId) throws DriverException;

    /**
     * Inserts a row at the end of the dataware with the specified values
     *
     * @param values Values of the inserted row fields in the field order
     *
     * @throws DriverException if the row could not be inserted
     */
    public void insertFilledRow(Value[] values) throws DriverException;

    /**
     * Inserts a row at the end of the dataware
     *
     * @param pk primary key of the inserted row
     *
     * @throws DriverException if the row could not be inserted
     */
    public void insertEmptyRow(ValueCollection pk) throws DriverException;

    /**
     * Begins a transaction
     *
     * @throws DriverException If the transaction could not be  started
     */
    public void beginTrans() throws DriverException;

    /**
     * Commits the changes made during the transaction
     *
     * @throws DriverException If the transaction could not  be commited
     */
    public void commitTrans() throws DriverException;

    /**
     * Cancels the changes made during the transaction
     *
     * @throws DriverException If the transaction could not  be cancelled
     */
    public void rollBackTrans() throws DriverException;

    /**
     * Sets the value of a cell of the table. Cannot be called outside a
     * beginTrans-commintTrans or beginTrans-rollBackTrans
     *
     * @param row row to update
     * @param fieldId field to update
     * @param value Value to update
     *
     * @throws DriverException If the operation failed
     */
    public void setFieldValue(long row, int fieldId, Value value)
        throws DriverException;
}
