package org.gvsig.remoteClient.gml.factories;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.gvsig.remoteClient.gml.schemas.IXMLType;
import org.gvsig.remoteClient.gml.schemas.XMLComplexType;
import org.gvsig.remoteClient.gml.schemas.XMLElement;
import org.gvsig.remoteClient.gml.schemas.XMLSchemaParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: XMLElementsFactory.java,v 1.1.2.4 2006/11/20 08:58:17 jorpiell Exp $
 * $Log: XMLElementsFactory.java,v $
 * Revision 1.1.2.4  2006/11/20 08:58:17  jorpiell
 * Una excepcin no capturada
 *
 * Revision 1.1.2.3  2006/11/17 11:38:03  ppiqueras
 * Corregidos bugs y añadida nueva funcionalidad. Del HEAD.
 *
 * Revision 1.3  2006/11/16 13:27:12  jorpiell
 * El getElement les asocia al los hijos el padre correspondiente
 *
 * Revision 1.2  2006/10/10 12:52:28  jorpiell
 * Soporte para features complejas.
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * The XML "element" is an XML tag of a XSD schema and represent
 * a simple "element" or "object". One element has its name and 
 * the data type that contains (and the data)
 * 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class XMLElementsFactory {
	private static Hashtable elements = new Hashtable();
	
	public static XMLElement getElement(String name){
		if (name == null){
			return null;
		}
		XMLElement element = (XMLElement)elements.get(name.toUpperCase());
		setParentNode(element);
		return element;
	}
	
	private static void setParentNode(XMLElement element){
		if (element.getEntityType() != null){
			if (element.getEntityType().getType() == IXMLType.COMPLEX){
				Vector attributes = ((XMLComplexType)element.getEntityType()).getAttributes();
				for (int i=0 ; i<attributes.size() ; i++){
					XMLElement child = (XMLElement)attributes.get(i);
					child.setParentElement(element);
					setParentNode(child);
				}
			}	
		}
	}
	
	/**
	 * It parses the element and register it
	 * @param schema
	 * Schema parser that contains a "element"
	 * @return
	 */
	public static XMLElement addType(XMLSchemaParser schema){
		XMLElement element;
		try {
			element = new XMLElement(schema);
			if (element.getName() != null){
				elements.put(element.getName().toUpperCase(),element);
			}
			return element;
		} catch (XmlPullParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * Just for degug. It prints all the registred components.
	 */
	public static void printEntities(){
		System.out.println("*** ELEMENTOS ***");
		Object[] keys = elements.keySet().toArray();
		for (int i=0 ; i<elements.size() ; i++){
			XMLElement entity = (XMLElement)elements.get(keys[i]);
			printEntity(entity,0);			
		}
	}
	
	public static void printEntity(XMLElement entity,int level){
		String tab = "";
		for (int i=0 ; i<level ; i++){
			tab = tab + "\t";
		}
		System.out.print(tab + "NAME: " + entity.getName());
		if (entity.getEntityType() != null){
			System.out.print(" TYPE: " + entity.getEntityType().getName() + "\n");
			if (entity.getEntityType().getType() == IXMLType.COMPLEX){
				Map children = ((XMLComplexType)entity.getEntityType()).getSubtypes();
				Set childrenKeys = children.keySet();
				Iterator it = childrenKeys.iterator();
				level++;
				while(it.hasNext()){
					String child = (String)it.next();
					XMLElement eChild = (XMLElement)children.get(child);
					printEntity(eChild,level);	
				}
			}
		}else{
			System.out.print(" TYPE: ERROR \n");
		}
		
	}
	
}
