package org.gvsig.remoteClient.gml.factories;

import java.util.Hashtable;
import java.util.StringTokenizer;

import org.gvsig.remoteClient.gml.GMLException;
import org.gvsig.remoteClient.gml.schemas.XMLSchema;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: XMLSchemasFactory.java,v 1.1 2006/08/10 12:00:49 jorpiell Exp $
 * $Log: XMLSchemasFactory.java,v $
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * Factory to create Schemas. The XML schema parser
 * adds a new schema every time it find one when is
 * parsing a xml (gml) file
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class XMLSchemasFactory {
	private static Hashtable schemas = new Hashtable();
	
	public static XMLSchema getType(String name){
		return (XMLSchema)schemas.get(name);
	}

	/**
	 * Adds a new Schema
	 * @param xmlnsName
	 * XML nameSpace name
	 * @param xmlnsValue
	 * XML nameSpace value
	 */
	public static void addType(String xmlnsName,String xmlnsValue){
		XMLSchema schema = new XMLSchema(xmlnsName,xmlnsValue);
		schemas.put(xmlnsName,schema);
	}	
	
	/**
	 * It find a nameSpace by location. 
	 * @param namespace
	 * Parent nameSpace
	 * @param location
	 * URL or file where the nameSpace is located
	 * @param schema
	 * Schema
	 * @return
	 * @throws GMLException
	 */
	public static String addSchemaLocation(String namespace,String location,String schema) throws GMLException{
		XMLSchema parentSchema = getType(namespace);
		XMLSchema childSchema = getSchemaByLocation(location);
		childSchema.setLocation(schema);
		childSchema.setParent(parentSchema);
		if (parentSchema != null){
			parentSchema.addChidren(childSchema);
		}
		return childSchema.getName();		
	}
	
	/**
	 * It find a nameSpace by location. 
	 * @param location
	 * URL or file where the nameSpace is located
	 * @return
	 */
	public static XMLSchema getSchemaByLocation(String location){
		Object[] keys = schemas.keySet().toArray();
		for (int i=0 ; i<schemas.size() ; i++){
			XMLSchema schema = (XMLSchema)schemas.get(keys[i]);
			if (schema.getLocation().equals(location)){
				return schema;
			}
		}
		return null;
	}
	
	/**
	 * Just for degug. It prints all the registred components.
	 */
	public static void printSchemas(){
		System.out.println("*** SCHEMAS ***");
		Object[] keys = schemas.keySet().toArray();
		for (int i=0 ; i<schemas.size() ; i++){
			XMLSchema schema = (XMLSchema)schemas.get(keys[i]);
			System.out.print("NAME: " + schema.getName());
			System.out.print(" LOCATION: " + schema.getLocation());
			if (schema.getChildren().size() > 0){
				for (int j=0 ; j< schema.getChildren().size() ; j++){
					System.out.print(" CHIDREN " + j + ": " + ((XMLSchema)schema.getChildren().get(j)).getName());
				}
			}
			System.out.print("\n");
		}
	}
}
