package org.gvsig.remoteClient.gml.schemas;

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.Hashtable;
import java.util.StringTokenizer;

import org.gvsig.remoteClient.gml.GMLException;
import org.gvsig.remoteClient.gml.GMLTags;
import org.gvsig.remoteClient.gml.factories.XMLSchemasFactory;
import org.gvsig.remoteClient.utils.Utilities;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informaci�n Geogr�fica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ib��ez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: XMLSchemasManager.java,v 1.2 2006/08/30 10:47:36 jorpiell Exp $
 * $Log: XMLSchemasManager.java,v $
 * Revision 1.2  2006/08/30 10:47:36  jorpiell
 * Añadido un File.separator en lugar de unas barras que impedia que funcioanra en linux
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * This class represents a GML file header. It has 
 * methods to parses the GML file header and retrieve
 * the namespaces and the attributes. If there is an schema
 * on the GML file, it has to manage it. It has to retrieve
 * the GML file version
 * 
 * @author Jorge Piera Llodr� (piera_jor@gva.es)
 */
public class XMLSchemasManager {
	private String version = null;
	private File file = null;
	private String targetNameSpace = null;
	
	public XMLSchemasManager(File file) {
		super();
		this.file = file;
	}
	
	/**
	 * It parses the GML header and returns the attributes
	 * @param parser
	 * Reader for the GML file 
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 * @throws GMLException 
	 */
	public void parse(XMLSchemaParser parser) throws XmlPullParserException, IOException, GMLException{
		parser.nextTag();
		
		for (int i=0 ; i<parser.getAttributeCount() ; i++){
			String attName = parser.getAttributeName(i);
			String attValue = parser.getAttributeValue(i);
			if (attName.compareTo(GMLTags.VERSION)==0){
				version = attValue;
				break;
			}
			if (attName.compareTo(GMLTags.XML_TARGET_NAMESPACE)==0){
				targetNameSpace = attValue;
				break;
			}
			String[] ns = attName.split(":");
			if ((ns.length>1) && (ns[0].compareTo(GMLTags.XML_NAMESPACE)==0)){
				parseNameSpace(ns[1],attValue);				
			}
			if ((ns.length>1) && (ns[1].compareTo(GMLTags.XML_SCHEMA_LOCATION)==0)){
				parseSchemaLocation(ns[0],attValue);
			}			
		}	
		
	}
	
	/**
	 * It parses an XML namespace tag
	 * @param sNameSPace
	 */
	private void parseNameSpace(String xmlnsName,String xmlnsValue){
		XMLSchemasFactory.addType(xmlnsName,xmlnsValue);		
	}
	
	/**
	 * Parses the schema location attribute
	 * @param schemas
	 * XML attribute than contain the schema location info
	 * @throws GMLException 
	 */
	private void parseSchemaLocation(String namespace,String schemas) throws GMLException{
		StringTokenizer tokenizer = new StringTokenizer(schemas, " \t");
        while (tokenizer.hasMoreTokens()){
            String parent = tokenizer.nextToken();
            if (!tokenizer.hasMoreTokens()){
                throw new GMLException(GMLException.EXC_NO_SCHEMA);
            }
            String schema = tokenizer.nextToken();
            String name = XMLSchemasFactory.addSchemaLocation(namespace,parent,schema);
            parseSchema(schema,name);             
        }
	}	
	
	/**
	 * It Downloads the schemas and parses them
	 * @param urlString
	 * Schema to parse
	 * @return
	 */
	private void parseSchema(String urlString,String nameSpace){
		if (urlString.indexOf("http://") != 0){
			File f = new File(urlString);
			if (!(f.isAbsolute())){
				urlString = file.getParentFile().getAbsolutePath() + File.separator +  urlString;
				f = new File(urlString);
			}
			new XMLSchemaParser().parse(f,nameSpace);
		}else{
			URL url;
			try {
				url = new URL(urlString);
				File f = Utilities.downloadFile(url,"gml_schmema.xsd", null);
				if (f == null){
					return;
				}else{
					new XMLSchemaParser().parse(f,nameSpace);
				}  
			}catch (MalformedURLException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();				
			} catch (ConnectException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (UnknownHostException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}            
		}		
	}

	/**
	 * @return Returns the targetNameSpace.
	 */
	public String getTargetNameSpace() {
		return targetNameSpace;
	}

	/**
	 * @return Returns the version.
	 * TODO: Manage the different versions
	 */
	public String getVersion() {
		if (version == null){
			return "2.1.2";
		}
		return version;
	}

}
