package org.gvsig.gui.beans.filterPanel;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JPanel;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;

import org.gvsig.gui.beans.filterPanel.filterButtons.FilterButtonsJPanel;
import org.gvsig.gui.beans.Messages;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * This abstract class represents the common components of the FilterQuery panels
 * 
 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public abstract class AbstractFilterQueryJPanel extends JPanel implements Serializable {
	public static final int DefaultHeight = 280;
	public static final int DefaultWidth = 490; 
	
	protected final int fieldsJPanelHeight = 135;
	protected final int fieldsJPanelWidth = 145;
	protected final int valuesJPanelHeight = fieldsJPanelHeight;
	protected final int valuesJPanelWidth = fieldsJPanelWidth;
	protected final int defaultBottomJPanelWidth = 480;
	protected final int defaultBottomJPanelHeight = 120;
	protected int filterJScrollPaneHeight;
	protected int filterJScrollPanelWidth;
	protected final int filterButtonsPanelHeight = FilterButtonsJPanel.default_FilterButtonsJPanellHeight;
	protected final int filterButtonsPanelWidth = FilterButtonsJPanel.default_FilterButtonsJPanelWidth;
	protected final int defaultTopJPanelWidth = defaultBottomJPanelWidth;
	protected final int defaultTopJPanelHeight = 135;
	protected int fieldsAndValuesJScrollPaneHeight = 100;
	protected int fieldsAndValuesJScrollPaneWidth = fieldsJPanelWidth;
	
	protected javax.swing.JLabel fieldsJLabel = null;
	protected javax.swing.JLabel valuesJLabel = null;
	protected javax.swing.JPanel fieldsJPanel = null;
	protected javax.swing.JPanel valuesJPanel = null;
	protected FilterButtonsJPanel filterButtonsJPanel = null;
	protected javax.swing.JScrollPane filterJScrollPane = null;
	protected javax.swing.JPanel topJPanel = null;
	protected javax.swing.JPanel bottomJPanel = null;
	protected javax.swing.JTextArea txtExpression = null;
	protected javax.swing.JTree fieldsJTree = null;
	protected javax.swing.JList valuesJList = null;
	protected javax.swing.JScrollPane fieldsJScrollPane = null;
	protected javax.swing.JScrollPane valuesJScrollPane = null;		

	protected String title;
	
	protected DefaultTreeModel defaultTreeModel;
	protected DefaultListModel valuesListModel;
	
	
	/**
	 * This is the default constructor
	 */
	public AbstractFilterQueryJPanel(String _title) {		
		super();
		title = _title;
	}
	/**
	 * This is the default constructor
	 */
	public AbstractFilterQueryJPanel() {		
		super();
	}
	
	/**
	 * This method initializes this
	 */
	protected void initialize() {
		this.setPreferredSize(new Dimension(DefaultWidth, DefaultHeight));
		
		this.setLayout(new GridBagLayout());
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;

		gridBagConstraints.anchor = GridBagConstraints.NORTH;
		this.add(getTopJPanel(), gridBagConstraints);

		gridBagConstraints.anchor = GridBagConstraints.SOUTH;
		this.add(getBottomJPanel(), gridBagConstraints);
	}
	
	/**
	 * This method initializes topJPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected javax.swing.JPanel getTopJPanel() {
		if (topJPanel == null) {
			topJPanel = new JPanel();

		    topJPanel.setPreferredSize(new Dimension(defaultTopJPanelWidth, defaultTopJPanelHeight));
		    topJPanel.setLayout(new GridBagLayout());
		    
		    GridBagConstraints gridBagConstraints = new GridBagConstraints();
		    gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;	    
		    
		    gridBagConstraints.anchor = GridBagConstraints.WEST;
		    topJPanel.add(getFieldsJPanel(), gridBagConstraints);
		    
			gridBagConstraints.anchor = GridBagConstraints.CENTER;
			topJPanel.add(getFilterButtonsJPanel(), gridBagConstraints);
			
			gridBagConstraints.anchor = GridBagConstraints.EAST;
			topJPanel.add(getValuesJPanel(), gridBagConstraints);
		}
		
		return topJPanel;
	}
	
	/**
	 * This method initializes bottomJPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected abstract JPanel getBottomJPanel();	
	
	/**
	 * This method initializes fieldsJLabel
	 *
	 * @return javax.swing.JLabel
	 */
	protected javax.swing.JLabel getFieldsJLabel() {
		if (fieldsJLabel == null) {
			fieldsJLabel = new javax.swing.JLabel();
			fieldsJLabel.setText(Messages.getText("Campos") + ":");
		}

		return fieldsJLabel;
	}	
	
	/**
	 * This method initializes fieldsJPanel
	 *
	 * @return javax.swing.JPanel
	 */
	protected abstract JPanel getFieldsJPanel();

	/**
	 * This method initializes valuesJLabel
	 *
	 * @return javax.swing.JLabel
	 */
	protected javax.swing.JLabel getValuesJLabel() {
		if (valuesJLabel == null) {
			valuesJLabel = new javax.swing.JLabel();
			valuesJLabel.setText(Messages.getText("Valores") + ":");
		}

		return valuesJLabel;
	}
	
	/**
	 * This method initializes valuesJPanel
	 *
	 * @return javax.swing.JPanel
	 */
	protected abstract JPanel getValuesJPanel();
	
	/**
	 * This method initializes filterJScrollPane
	 *
	 * @return javax.swing.JScrollPane
	 */
	protected abstract javax.swing.JScrollPane getFilterJScrollPane();
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param symbol DOCUMENT ME!
	 */
	protected void putSymbol(String symbol) {
		int position = txtExpression.getCaretPosition();
		
		txtExpression.setText(insert(txtExpression.getText(), position, symbol));

		if (symbol.equals(" () ")) {
			position = position + 2;
		} else {
			position = position + symbol.length();
		}

		txtExpression.setCaretPosition(position);
	}
	
	/**
	 * This method initializes fieldsJTree
	 *
	 * @return javax.swing.JTree
	 */
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getLstCampos()
	 */
	protected javax.swing.JTree getFieldsJTree() {
		if (fieldsJTree == null) {
			fieldsJTree = new javax.swing.JTree(new Vector(0,1));
			
			// Remove icons:
			DefaultTreeCellRenderer defaultTreeCellRenderer = new DefaultTreeCellRenderer();
			defaultTreeCellRenderer.setOpenIcon(null);
			defaultTreeCellRenderer.setClosedIcon(null);
			defaultTreeCellRenderer.setLeafIcon(null);
			
			// Root not visible
			fieldsJTree.setRootVisible(false);
			fieldsJTree.setCellRenderer(defaultTreeCellRenderer);
		}

		return fieldsJTree;
	}
	
	/**
	 * This method initializes valuesJList
	 *
	 * @return javax.swing.JList
	 */
	protected javax.swing.JList getValuesJList() {
		if (valuesJList == null) {
			valuesJList = new javax.swing.JList(new DefaultListModel());
			valuesListModel = new DefaultListModel();
			valuesJList.setModel(valuesListModel);
			valuesJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
			
//			valuesJList.addMouseListener(new MouseAdapter() {
//				/*
//				 *  (non-Javadoc)
//				 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
//				 */
//				public void mouseClicked(MouseEvent e) {
//					if (e.getClickCount() == 2) {
//						int row = fieldsJTree.getRowForLocation(e.getX(), e.getY());
//	
//						if (row > -1) {
//							putSymbol(((DefaultListModel)valuesJList.getModel()).get(row).toString());
//						}
//					}
//				}
//			});
		}

		return valuesJList;
	}
	
    /**
     * Inserta una string en otra en la posicin indicada
     *
     * @param base String donde se inserta
     * @param position posicin de "base" donde se inserta la String
     * @param injerto String que se inserta en "base"
     *
     * @return String con la insercin hecha
     */
    protected static String insert(String base, int position, String injerto) {
        return base.substring(0, position) + injerto +
        base.substring(position);
    }
    
	
	/**
	 * This method initializes filterButtonsJPanel
	 *
	 * @return javax.swing.JPanel
	 */
	protected JPanel getFilterButtonsJPanel() {
		if (filterButtonsJPanel == null) {
			filterButtonsJPanel = new FilterButtonsJPanel();
			filterButtonsJPanel.setPreferredSize(new Dimension(filterButtonsPanelWidth, filterButtonsPanelHeight));
			filterButtonsJPanel.addPropertyChangeListener(new PropertyChangeListener() {
				/*
				 *  (non-Javadoc)
				 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
				 */
				public void propertyChange(PropertyChangeEvent arg0) {
					if (arg0.getPropertyName().equals(FilterButtonsJPanel.BUTTON_CLICKED_ACTION_COMMAND)) {
		            	
						switch(Integer.parseInt(arg0.getNewValue().toString()))	{
							case FilterButtonsJPanel.AND:
								putSymbol(" and ");
								break;
							case FilterButtonsJPanel.DATE:
								putSymbol( ((FilterButtonsJPanel)arg0.getSource()).getLastSelectedDate() );
								break;
							case FilterButtonsJPanel.DISTINCT:
								putSymbol(" <> ");
								break;
							case FilterButtonsJPanel.EQUAL:
								putSymbol(" = ");
								break;
							case FilterButtonsJPanel.EQUALGREATER:
								putSymbol(" >= ");
								break;
							case FilterButtonsJPanel.EQUALSMALLER:
								putSymbol(" <= ");
								break;
							case FilterButtonsJPanel.GREATER:
								putSymbol(" > ");
								break;
							case FilterButtonsJPanel.NOT:
								putSymbol(" not ");
								break;
							case FilterButtonsJPanel.OR:
								putSymbol(" or ");
								break;
							case FilterButtonsJPanel.PARENTHESIS:
								putSymbol(" () ");
								break;
							case FilterButtonsJPanel.SMALLER:
								putSymbol(" < ");
								break;
							case FilterButtonsJPanel.DELETE_TEXT:
								txtExpression.setText("");
								break;
							default: // do anything
				        }
					}
				}			
			});
		}
		
		return filterButtonsJPanel;
	}
	
	/**
	 * Sets new height to the 'topJPanel', (new Height must be bigger than default, else do nothing)
	 * 
	 * @param new_Height New height
	 */
	public void resizeHeight(int new_Height) {
		int difference = new_Height - DefaultHeight;
		
		if (difference > 0) {
			this.setPreferredSize(new Dimension(this.getPreferredSize().width, this.getPreferredSize().height + difference));
			topJPanel.setPreferredSize(new Dimension(topJPanel.getPreferredSize().width, topJPanel.getPreferredSize().height + difference));
			
			fieldsJPanel.setPreferredSize(new Dimension(fieldsJPanel.getPreferredSize().width, fieldsJPanel.getPreferredSize().height + difference));
			fieldsJScrollPane.setPreferredSize(new Dimension(fieldsJScrollPane.getPreferredSize().width, fieldsJScrollPane.getPreferredSize().height + difference));
		
			valuesJPanel.setPreferredSize(new Dimension(valuesJPanel.getPreferredSize().width, valuesJPanel.getPreferredSize().height + difference));
			valuesJScrollPane.setPreferredSize(new Dimension(valuesJScrollPane.getPreferredSize().width, valuesJScrollPane.getPreferredSize().height + difference));
		}
	}
	
	/**
	 * Sets the width to this JPanel
	 * 
	 * @param new_Width New width
	 */
	public abstract void resizeWidth(int new_Width);
	
	/*
	 *  (non-Javadoc)
	 * @see java.awt.Component#resize(int, int)
	 */
	public void resize(int width, int height) {
		int difference = height - DefaultHeight;

		if (difference != 0)
			this.resizeHeight(height);
		
		this.resizeWidth(width);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see java.awt.Component#resize(java.awt.Dimension)
	 */
	public void resize(Dimension d) {
		int difference = d.height - DefaultHeight;

		if (difference != 0)
			this.resizeHeight(d.height);
		
		this.resizeWidth(d.width);
	}
}
