package org.gvsig.gui.beans.filterPanel.filterQueryPanel;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.Serializable;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.gvsig.gui.beans.Messages;
import org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * This class is a version of the graphical interface of "FilterDialog" which was made by Fernando and is in 'appgvSIG' project.
 * It's supposed that other classes will extend from this and will add the logic as is in the "FilterDialog" class.
 * 
 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public class FilterQueryJPanel extends AbstractFilterQueryJPanel implements Serializable{
	private static final long serialVersionUID = -5772343767095703604L;

	protected int filterJScrollPaneHeight = defaultBottomJPanelHeight - 10;
	protected int filterJScrollPanelWidth = defaultBottomJPanelWidth;
	
	/**
	 * @see AbstractFilterQueryJPanel#AbstractFilterQueryJPanel()
	 */
	public FilterQueryJPanel() {
		super();
		this.initialize();
	}

	/**
	 * @see AbstractFilterQueryJPanel#AbstractFilterQueryJPanel(String)
	 */
	public FilterQueryJPanel(String _title) {
		super(_title);
		this.initialize();
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#initialize()
	 */
	protected void initialize() {
		super.initialize();
		this.setBorder(BorderFactory.createTitledBorder(Messages.getText("filterOnAWFSLayer")));
	}
	
	/*
	 *  (FilterQueryJPanel)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getFieldsJPanel()
	 */
	protected JPanel getFieldsJPanel() {
		if (fieldsJPanel == null) {
			fieldsJPanel = new javax.swing.JPanel();
			fieldsJPanel.setPreferredSize(new java.awt.Dimension(fieldsJPanelWidth, fieldsJPanelHeight));
			fieldsJPanel.setLayout(new GridBagLayout());
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
			gridBagConstraints.anchor = GridBagConstraints.NORTH;			
			fieldsJPanel.add(getFieldsJLabel(), gridBagConstraints);
			gridBagConstraints.anchor = GridBagConstraints.SOUTH;
			fieldsJPanel.add(getFieldsJScrollPane(), gridBagConstraints);
			
		}

		return fieldsJPanel;
	}

	/**
	 * This method initializes jScrollPane
	 *
	 * @return javax.swing.JScrollPane
	 */
	protected javax.swing.JScrollPane getFieldsJScrollPane() {
		if (fieldsJScrollPane == null) {
			fieldsJScrollPane = new javax.swing.JScrollPane();
			fieldsJScrollPane.setPreferredSize(new java.awt.Dimension(fieldsAndValuesJScrollPaneWidth, fieldsAndValuesJScrollPaneHeight));
			fieldsJScrollPane.setViewportView(getFieldsJTree());
		}

		return fieldsJScrollPane;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getValuesJPanel()
	 */
	protected JPanel getValuesJPanel() {
		if (valuesJPanel == null) {
			valuesJPanel = new javax.swing.JPanel();
			valuesJPanel.setPreferredSize(new java.awt.Dimension(valuesJPanelWidth, valuesJPanelHeight));
			valuesJPanel.setLayout(new GridBagLayout());
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
			gridBagConstraints.anchor = GridBagConstraints.NORTH;
			valuesJPanel.add(getValuesJLabel(), gridBagConstraints);
			gridBagConstraints.anchor = GridBagConstraints.SOUTH;
			valuesJPanel.add(getValuesJScrollPane(), gridBagConstraints);
		}

		return valuesJPanel;
	}
	
	/**
	 * This method initializes jScrollPane1
	 *
	 * @return javax.swing.JScrollPane
	 */
	protected javax.swing.JScrollPane getValuesJScrollPane() {
		if (valuesJScrollPane == null) {
			valuesJScrollPane = new javax.swing.JScrollPane();
			valuesJScrollPane.setPreferredSize(new Dimension(fieldsAndValuesJScrollPaneWidth, fieldsAndValuesJScrollPaneHeight));
			valuesJScrollPane.setViewportView(getValuesJList());
		}

		return valuesJScrollPane;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getFilterJScrollPane()
	 */
	protected JScrollPane getFilterJScrollPane() {
		if (filterJScrollPane == null) {
			filterJScrollPane = new javax.swing.JScrollPane();
			filterJScrollPane.setPreferredSize(new java.awt.Dimension(filterJScrollPanelWidth, filterJScrollPaneHeight));
			filterJScrollPane.setViewportView(getTxtExpression());
			filterJScrollPane.setHorizontalScrollBarPolicy(javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		}

		return filterJScrollPane;
	}	

	/**
	 * This method initializes txtExpression
	 *
	 * @return javax.swing.JTextArea
	 */
	protected javax.swing.JTextArea getTxtExpression() {
		if (txtExpression == null) {
			txtExpression = new javax.swing.JTextArea();
			txtExpression.setLineWrap(true);
		}

		return txtExpression;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getBottomJPanel()
	 */
	protected JPanel getBottomJPanel() {
		if (bottomJPanel == null) {
			bottomJPanel = new JPanel();
			bottomJPanel.setPreferredSize(new Dimension(defaultBottomJPanelWidth, defaultBottomJPanelHeight));
			bottomJPanel.add(getFilterJScrollPane(), null);
		}
		
		return bottomJPanel;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#resizeWidth(int)
	 */
	public void resizeWidth(int new_Width) {	
		int difference = new_Width - DefaultWidth;
		
		if (difference != 0) {
			this.setPreferredSize(new Dimension(this.getPreferredSize().width + difference, this.getPreferredSize().height));
			topJPanel.setPreferredSize(new Dimension(topJPanel.getPreferredSize().width + difference, topJPanel.getPreferredSize().height));
			bottomJPanel.setPreferredSize(new Dimension(bottomJPanel.getPreferredSize().width + difference, bottomJPanel.getPreferredSize().height));
			filterJScrollPane.setPreferredSize(new Dimension(filterJScrollPane.getPreferredSize().width + difference, filterJScrollPane.getPreferredSize().height));
			
			int halfDifference = difference / 2;
			
			fieldsJPanel.setPreferredSize(new Dimension(fieldsJPanel.getPreferredSize().width + halfDifference, fieldsJPanel.getPreferredSize().height));
			fieldsJScrollPane.setPreferredSize(new Dimension(fieldsJScrollPane.getPreferredSize().width + halfDifference, fieldsJScrollPane.getPreferredSize().height));
		
			valuesJPanel.setPreferredSize(new Dimension(valuesJPanel.getPreferredSize().width + halfDifference, valuesJPanel.getPreferredSize().height));
			valuesJScrollPane.setPreferredSize(new Dimension(valuesJScrollPane.getPreferredSize().width + halfDifference, valuesJScrollPane.getPreferredSize().height));
		}
	}
}
