package org.gvsig.gui.beans.textBoxWithCalendar;

import java.awt.Dimension;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import javax.swing.JDialog;

import org.freixas.jcalendar.DateEvent;
import org.freixas.jcalendar.DateListener;
import org.freixas.jcalendar.JCalendar;
import org.gvsig.gui.beans.Messages;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * Creates a Dialog that allows users to select the date they want
 * This class is a version of JCalendarDatePanel
 * 
 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public class JCalendarDateDialog extends JDialog implements Serializable{
	private static final long serialVersionUID = 3254428202468985837L;
	private JCalendar jCalendar = null;
	private int defaultWidth = 120;
	private int defaultHeight = 19;

	/**
	 * Default Constructor
	 */
	public JCalendarDateDialog() {
		super();
		try
		{
			this.setSize(new Dimension(defaultWidth, defaultHeight));
			this.initialize();			
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * This is the default constructor with 2 parameters for set the size
	 */
	public JCalendarDateDialog(int width, int height) {
		super();
		try
		{			
			defaultWidth = width;
			defaultHeight = height;
			
			this.setSize(new Dimension(defaultWidth, defaultHeight));
			this.initialize();			
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * This method initializes this 
	 */
	private void initialize() {
		this.setDefaultTitle();
		
		// Adds the jCalendar to this dialog
		this.getContentPane().add(this.getJCalendar());
	}

	/**
	 * Sets the size of this panel
	 * 
	 * @param width (the new Width for the panel)
	 * @param height (the new Height for the panel)
	 */
	public void setSizeResize(int width, int height)
	{
		this.defaultWidth = width;
		this.defaultHeight = height;
		this.setSize(new Dimension(defaultWidth, defaultHeight));
		getJCalendar().revalidate();
	}
	
	/**
	 * Get the height of this panel
	 */
	public int getHeight()
	{
		return this.defaultHeight;
	}
	
	/**
	 * Get the width of this panel
	 */
	public int getWidth()
	{
		return this.defaultWidth;
	}
	
	/**
	 * This method initializes jCalendar	
	 * 	
	 * @return javax.swing.JCalendar	
	 */
	private JCalendar getJCalendar() {
		if (jCalendar == null) {
			jCalendar = new JCalendar();
			jCalendar.setToolTipText(Messages.getText("calendarSelectDate"));
			
			try
			{
				jCalendar.setToolTipTextToMonthDecrButton(Messages.getText("calendarBackOneMonth"));
				jCalendar.setToolTipTextToMonthIncrButton(Messages.getText("calendarForwardOneMonth"));
				jCalendar.setToolTipTextToYearDecrButton(Messages.getText("calendarBackOneYear"));
				jCalendar.setToolTipTextToYearIncrButton(Messages.getText("calendarForwardOneYear"));

				// Adds a date listener calendar
				jCalendar.addDateListener(new DateListener(){
					/*
					 *  (non-Javadoc)
					 * @see org.freixas.jcalendar.DateListener#dateChanged(org.freixas.jcalendar.DateEvent)
					 */
					public void dateChanged(DateEvent arg0) {
						hide();
					}				
				});
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
		return jCalendar;
	}

	/**
	 * Gets the date of the calendar (default or last selected)
	 * 
	 * @return Date
	 */
	public Date getDate() {		
		return jCalendar.getDate();
	}

	/**
	 * Sets the date to the calendar
	 * 
	 * @param Date
	 */
	public void setDate(Date date)
	{
		jCalendar.setDate(date);
	}

	/**
	 * Returns the date formatted
	 * 
	 * @param Date
	 * @return String The formatted date
	 */
	private String getFormattedDate(Date d) {
		return new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault()).format(d);
	}
	
	/**
	 * Returns the date selected, formatted
	 * 
	 * @return String The formatted date
	 */
	public String getFormattedDate() {
		return new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault()).format(this.getDate());
	}

	/**
	 * Gets the title of the JDialog with the calendar component. The title is displayed in the JDialog's border.
	 *
	 * @return Title
	 */
	public String getTitle() {
		return super.getTitle();
	}
	
	/**
	 * Sets the title of the JDialog with the calendar component. 
	 * 
	 * @param String
	 */
	public void setTitle(String title) {
		super.setTitle(title);
	}

	/**
	 * Sets the default title of the JDialog with the calendar component.
	 */
	public void setDefaultTitle() {
		this.setTitle(Messages.getText("calendarTitle"));
	}
	
	/**
	 * @see java.awt.Dialog#setModal(boolean)
	 */
	public void setModal(boolean b) {
		super.setModal(b);
	}
}
